#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

import asyncio

import pytest

from featureform import canonicalize_function_definition

"""
DO NOT FORMAT THIS FILE
"""

# fmt: off


def my_function1(
        df
):
    # This is a comment
    df = df.head(5)  # Take top 5 rows
    if df.empty:  # Check if empty

        return df  # Return empty dataframe
    return df  # Final return


expected_output1 = """def my_function1(df):
    df = df.head(5)
    if df.empty:
        return df
    return df"""


def my_function2(df):
    # This function processes a dataframe
    """
    some crazy doc string
    :param df:
    :return:
    """
    df = df.dropna()
    # Remove missing values
    if df.empty:  # Check if DataFrame is empty
        return df  # Return the empty dataframe
    return df  # Returning the dataframe


expected_output2 = """def my_function2(df):
    df = df.dropna()
    if df.empty:
        return df
    return df"""


def my_function3(df):
    df = df[df['col'] > 0]  # Keep only positive values
    if (df.empty):  # Check if df is empty
        # If empty, return df
        return (df)
    df['new_col'] = df['col'] * 2  # Create a new column
    return df  # Return the modified dataframe


expected_output3 = """def my_function3(df):
    df = df[df['col'] > 0]
    if df.empty:
        return df
    df['new_col'] = df['col'] * 2
    return df"""


def my_function_with_decorator(df):
    @some_decorator
    def inner_function(x):
        return x * 2
    return inner_function(df)


expected_output_with_decorator = """def my_function_with_decorator(df):

    @some_decorator
    def inner_function(x):
        return x * 2
    return inner_function(df)"""


def my_function_with_nested(df):
    def helper(y):
        return y + 1
    return df.apply(helper)

expected_output_with_nested = """def my_function_with_nested(df):

    def helper(y):
        return y + 1
    return df.apply(helper)"""


def my_function_with_defaults(a, b=10):
    return a + b


expected_output_with_defaults = """def my_function_with_defaults(a, b=10):
    return a + b"""


def my_function_with_annotations(a: int, b: str) -> bool:
    return str(a) == b


expected_output_with_annotations = """def my_function_with_annotations(a: int, b: str) ->bool:
    return str(a) == b"""


def my_function_with_lambda(df):
    return df.apply(lambda x: x * 2)


expected_output_with_lambda = """def my_function_with_lambda(df):
    return df.apply(lambda x: x * 2)"""


def my_function_with_control_flow(df):
    try:
        df['new_col'] = df['col'] / df['divider']
    except ZeroDivisionError:
        df['new_col'] = None
    finally:
        return df


expected_output_with_control_flow = """def my_function_with_control_flow(df):
    try:
        df['new_col'] = df['col'] / df['divider']
    except ZeroDivisionError:
        df['new_col'] = None
    finally:
        return df"""


def my_function_with_multiline_string():
    sql_query = """
    SELECT *
    FROM table
    WHERE condition
    """
    return sql_query


expected_output_with_multiline_string = '''def my_function_with_multiline_string():
    sql_query = """
    SELECT *
    FROM table
    WHERE condition
    """
    return sql_query'''

global_var = 0


def my_function_with_globals():
    global global_var
    global_var += 1
    return global_var


expected_output_with_globals = """def my_function_with_globals():
    global global_var
    global_var += 1
    return global_var"""


@pytest.mark.parametrize(
    "my_function, expected_output",
    [
        (my_function1, expected_output1),
        (my_function2, expected_output2),
        (my_function3, expected_output3),
        (my_function_with_decorator, expected_output_with_decorator),
        (my_function_with_nested, expected_output_with_nested),
        (my_function_with_defaults, expected_output_with_defaults),
        (my_function_with_annotations, expected_output_with_annotations),
        (my_function_with_lambda, expected_output_with_lambda),
        (my_function_with_control_flow, expected_output_with_control_flow),
        (my_function_with_multiline_string, expected_output_with_multiline_string),
        (my_function_with_globals, expected_output_with_globals),
    ],
)
def test_canonicalize_function_definition(my_function, expected_output):
    normalized_code = canonicalize_function_definition(my_function)
    assert normalized_code == expected_output

# fmt: on



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

import pytest
from featureform import FilePrefix
from featureform.enums import FileFormat, RefreshMode, Initialize
from featureform.proto import metadata_pb2 as pb


@pytest.mark.parametrize(
    "store_type, file_path",
    [
        ("AZURE", "abfss://test_bucket/test_file"),
        ("AZURE", "test_bucket/test_file"),
        ("S3", "s3://test_bucket/test_file"),
        ("S3", "test_bucket/test_file"),
        ("HDFS", "hdfs://test_bucket/test_file"),
        ("HDFS", "test_bucket/test_file"),
        ("GCS", "gs://test_bucket/test_file"),
        ("GCS", "test_bucket/test_file"),
    ],
)
def test_validate_file_scheme(store_type, file_path):
    try:
        FilePrefix.validate(store_type, file_path)
    except Exception as e:
        assert (
            str(e)
            == f"File path '{file_path}' must be a full path. Must start with '{FilePrefix[store_type].prefixes}'"
        )


@pytest.mark.parametrize(
    "location, expected_format",
    [
        ("s3://bucket/path/to/file.csv", "csv"),
        ("s3a://bucket/path/to/file.csv", "csv"),
        ("s3://bucket/path/to/directory/part-0000.parquet", "parquet"),
        ("s3://bucket/path/to/directory", "parquet"),
    ],
)
def test_get_format(location, expected_format):
    assert FileFormat.get_format(location, "parquet") == expected_format


@pytest.mark.parametrize(
    "input, output",
    [
        (pb.RefreshMode.REFRESH_MODE_AUTO, RefreshMode.AUTO),
        (pb.RefreshMode.REFRESH_MODE_FULL, RefreshMode.FULL),
        (pb.RefreshMode.REFRESH_MODE_INCREMENTAL, RefreshMode.INCREMENTAL),
        (pb.RefreshMode.REFRESH_MODE_UNSPECIFIED, None),
    ],
)
def test_refresh_mode_from_proto(input, output):
    assert RefreshMode.from_proto(input) == output


@pytest.mark.parametrize(
    "input, output",
    [
        (RefreshMode.AUTO, pb.RefreshMode.REFRESH_MODE_AUTO),
        (RefreshMode.FULL, pb.RefreshMode.REFRESH_MODE_FULL),
        (RefreshMode.INCREMENTAL, pb.RefreshMode.REFRESH_MODE_INCREMENTAL),
    ],
)
def test_refresh_mode_to_proto(input, output):
    assert RefreshMode.to_proto(input) == output


@pytest.mark.parametrize(
    "invalid_string_value, expected_message",
    [
        ("invalid_value", "Refresh Mode value not supported"),
        (None, "Refresh Mode value required: received"),
    ],
)
def test_refresh_mode_from_string_invalid_value(invalid_string_value, expected_message):

    with pytest.raises(ValueError, match=expected_message):
        RefreshMode.from_string(invalid_string_value)


@pytest.mark.parametrize(
    "valid_string_value, expected_enum",
    [
        ("AUTO", RefreshMode.AUTO),
        ("full", RefreshMode.FULL),
        ("INCREMENTAL", RefreshMode.INCREMENTAL),
    ],
)
def test_refresh_mode_from_string_valid_value(valid_string_value, expected_enum):

    result = RefreshMode.from_string(valid_string_value)
    assert result == expected_enum, f"Expected {expected_enum}, got {result}"


@pytest.mark.parametrize(
    "input, output",
    [
        (pb.Initialize.INITIALIZE_ON_CREATE, Initialize.ON_CREATE),
        (pb.Initialize.INITIALIZE_ON_SCHEDULE, Initialize.ON_SCHEDULE),
        (pb.Initialize.INITIALIZE_UNSPECIFIED, None),
    ],
)
def test_initialize_from_proto(input, output):
    assert Initialize.from_proto(input) == output


@pytest.mark.parametrize(
    "input, output",
    [
        (Initialize.ON_CREATE, pb.Initialize.INITIALIZE_ON_CREATE),
        (Initialize.ON_SCHEDULE, pb.Initialize.INITIALIZE_ON_SCHEDULE),
    ],
)
def test_initialize_to_proto(input, output):
    assert Initialize.to_proto(input) == output


@pytest.mark.parametrize(
    "invalid_string_value, expected_message",
    [
        ("invalid_value", "Initialize value not supported"),
        (None, "Initialize value required: received"),
    ],
)
def test_initialize_from_string_invalid_value(invalid_string_value, expected_message):

    with pytest.raises(ValueError, match=expected_message):
        Initialize.from_string(invalid_string_value)


@pytest.mark.parametrize(
    "valid_string_value, expected_enum",
    [
        ("ON_CREATE", Initialize.ON_CREATE),
        ("on_create", Initialize.ON_CREATE),
        ("ON_SCHEDULE", Initialize.ON_SCHEDULE),
        ("on_schedule", Initialize.ON_SCHEDULE),
    ],
)
def test_initialize_from_string_valid_value(valid_string_value, expected_enum):

    result = Initialize.from_string(valid_string_value)
    assert result == expected_enum, f"Expected {expected_enum}, got {result}"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: featureform/proto/metadata.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n featureform/proto/metadata.proto\x12\"featureform.serving.metadata.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x19google/protobuf/any.proto\"\x17\n\x15PassThroughAuthConfig\"\x14\n\x04Name\x12\x0c\n\x04name\x18\x01 \x01(\t\"Y\n\x0bNameRequest\x12\x36\n\x04name\x18\x01 \x01(\x0b\x32(.featureform.serving.metadata.proto.Name\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"S\n\x0b\x45rrorStatus\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12%\n\x07\x64\x65tails\x18\x03 \x03(\x0b\x32\x14.google.protobuf.Any\"\xad\x02\n\x0eResourceStatus\x12I\n\x06status\x18\x01 \x01(\x0e\x32\x39.featureform.serving.metadata.proto.ResourceStatus.Status\x12\x15\n\rerror_message\x18\x02 \x01(\t\x12\x45\n\x0c\x65rror_status\x18\x03 \x01(\x0b\x32/.featureform.serving.metadata.proto.ErrorStatus\"r\n\x06Status\x12\r\n\tNO_STATUS\x10\x00\x12\x0b\n\x07\x43REATED\x10\x01\x12\x0b\n\x07PENDING\x10\x02\x12\t\n\x05READY\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04\x12\x0b\n\x07RUNNING\x10\x05\x12\r\n\tCANCELLED\x10\x06\x12\x0c\n\x08\x44\x45LETING\x10\x07\"\x98\x01\n\nResourceID\x12\x41\n\x08resource\x18\x01 \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12G\n\rresource_type\x18\x02 \x01(\x0e\x32\x30.featureform.serving.metadata.proto.ResourceType\"\x9b\x01\n\x10SetStatusRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\"n\n\x15ScheduleChangeRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\x12\x10\n\x08schedule\x18\x02 \x01(\t\",\n\x0bNameVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\"o\n\x12NameVariantRequest\x12\x45\n\x0cname_variant\x18\x01 \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x07\n\x05\x45mpty\"!\n\x0bListRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"\x86\x01\n\x07\x46\x65\x61ture\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x17\n\x0f\x64\x65\x66\x61ult_variant\x18\x03 \x01(\t\x12\x10\n\x08variants\x18\x04 \x03(\t\"4\n\x07\x43olumns\x12\x0e\n\x06\x65ntity\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\n\n\x02ts\x18\x03 \x01(\t\"\x1f\n\x0ePythonFunction\x12\r\n\x05query\x18\x01 \x01(\x0c\"\"\n\x06Stream\x12\x18\n\x10offline_provider\x18\x01 \x01(\t\"\xdc\x02\n\x0fResourceVariant\x12M\n\x0f\x66\x65\x61ture_variant\x18\x01 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FeatureVariantH\x00\x12I\n\rlabel_variant\x18\x02 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.LabelVariantH\x00\x12V\n\x14training_set_variant\x18\x03 \x01(\x0b\x32\x36.featureform.serving.metadata.proto.TrainingSetVariantH\x00\x12K\n\x0esource_variant\x18\x04 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.SourceVariantH\x00\x42\n\n\x08resource\"p\n\x14GetEquivalentRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x44\n\x07variant\x18\x02 \x01(\x0b\x32\x33.featureform.serving.metadata.proto.ResourceVariant\"g\n\nRunRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x45\n\x08variants\x18\x02 \x03(\x0b\x32\x33.featureform.serving.metadata.proto.ResourceVariant\"\x99\n\n\x0e\x46\x65\x61tureVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12?\n\x06source\x18\x03 \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x0e\n\x06\x65ntity\x18\x05 \x01(\t\x12+\n\x07\x63reated\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05owner\x18\x07 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\t\x12\x10\n\x08provider\x18\t \x01(\t\x12\x42\n\x06status\x18\n \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x45\n\x0ctrainingsets\x18\x0b \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12>\n\x07\x63olumns\x18\x0c \x01(\x0b\x32+.featureform.serving.metadata.proto.ColumnsH\x00\x12\x46\n\x08\x66unction\x18\x11 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.PythonFunctionH\x00\x12<\n\x06stream\x18\x15 \x01(\x0b\x32*.featureform.serving.metadata.proto.StreamH\x00\x12\x30\n\x0clast_updated\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08schedule\x18\x0e \x01(\t\x12\x36\n\x04tags\x18\x0f \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x10 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x41\n\x04mode\x18\x12 \x01(\x0e\x32\x33.featureform.serving.metadata.proto.ComputationMode\x12\x13\n\x07task_id\x18\x17 \x01(\tB\x02\x18\x01\x12\x14\n\x0ctask_id_list\x18\x18 \x03(\t\x12T\n\x15\x61\x64\x64itional_parameters\x18\x16 \x01(\x0b\x32\x35.featureform.serving.metadata.proto.FeatureParameters\x12;\n\x04type\x18\x19 \x01(\x0b\x32-.featureform.serving.metadata.proto.ValueType\x12^\n\x19resource_snowflake_config\x18\x1a \x01(\x0b\x32;.featureform.serving.metadata.proto.ResourceSnowflakeConfig\x12\x16\n\nis_deleted\x18\x1b \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x1c \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\x12\x1e\n\x16offline_store_provider\x18\x1d \x01(\t\x12M\n\x17offline_store_locations\x18\x1e \x03(\x0b\x32,.featureform.serving.metadata.proto.LocationB\n\n\x08location\"x\n\x15\x46\x65\x61tureVariantRequest\x12K\n\x0f\x66\x65\x61ture_variant\x18\x01 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FeatureVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x97\x01\n\tValueType\x12@\n\x06scalar\x18\x01 \x01(\x0e\x32..featureform.serving.metadata.proto.ScalarTypeH\x00\x12@\n\x06vector\x18\x02 \x01(\x0b\x32..featureform.serving.metadata.proto.VectorTypeH\x00\x42\x06\n\x04Type\"u\n\nVectorType\x12>\n\x06scalar\x18\x01 \x01(\x0e\x32..featureform.serving.metadata.proto.ScalarType\x12\x11\n\tdimension\x18\x02 \x01(\x05\x12\x14\n\x0cis_embedding\x18\x03 \x01(\x08\"\xcf\x01\n\x11\x46\x65\x61tureParameters\x12W\n\x0bprecomputed\x18\x01 \x01(\x0b\x32@.featureform.serving.metadata.proto.PrecomputedFeatureParametersH\x00\x12Q\n\x08ondemand\x18\x02 \x01(\x0b\x32=.featureform.serving.metadata.proto.OndemandFeatureParametersH\x00\x42\x0e\n\x0c\x66\x65\x61ture_type\"\x1e\n\x1cPrecomputedFeatureParameters\"/\n\x19OndemandFeatureParameters\x12\x12\n\ndefinition\x18\x01 \x01(\t\"d\n\nFeatureLag\x12\x0f\n\x07\x66\x65\x61ture\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12&\n\x03lag\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\"\x84\x01\n\x05Label\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x17\n\x0f\x64\x65\x66\x61ult_variant\x18\x03 \x01(\t\x12\x10\n\x08variants\x18\x04 \x03(\t\"\xd6\x07\n\x0cLabelVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12?\n\x06source\x18\x05 \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x0e\n\x06\x65ntity\x18\x06 \x01(\t\x12+\n\x07\x63reated\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05owner\x18\x08 \x01(\t\x12\x10\n\x08provider\x18\t \x01(\t\x12\x42\n\x06status\x18\n \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x45\n\x0ctrainingsets\x18\x0b \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x42\n\x07\x63olumns\x18\x0c \x01(\x0b\x32+.featureform.serving.metadata.proto.ColumnsB\x02\x18\x01H\x00\x12<\n\x06stream\x18\x10 \x01(\x0b\x32*.featureform.serving.metadata.proto.StreamH\x00\x12M\n\x0f\x65ntity_mappings\x18\x16 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.EntityMappingsH\x00\x12\x36\n\x04tags\x18\r \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x0e \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x13\n\x07task_id\x18\x0f \x01(\tB\x02\x18\x01\x12;\n\x04type\x18\x12 \x01(\x0b\x32-.featureform.serving.metadata.proto.ValueType\x12\x14\n\x0ctask_id_list\x18\x11 \x03(\t\x12^\n\x19resource_snowflake_config\x18\x13 \x01(\x0b\x32;.featureform.serving.metadata.proto.ResourceSnowflakeConfig\x12\x16\n\nis_deleted\x18\x14 \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x15 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\x42\n\n\x08location\"\x85\x01\n\x0e\x45ntityMappings\x12\x43\n\x08mappings\x18\x01 \x03(\x0b\x32\x31.featureform.serving.metadata.proto.EntityMapping\x12\x14\n\x0cvalue_column\x18\x02 \x01(\t\x12\x18\n\x10timestamp_column\x18\x03 \x01(\t\"4\n\rEntityMapping\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rentity_column\x18\x02 \x01(\t\"r\n\x13LabelVariantRequest\x12G\n\rlabel_variant\x18\x01 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.LabelVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x8c\x05\n\x08Provider\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x10\n\x08software\x18\x04 \x01(\t\x12\x0c\n\x04team\x18\x05 \x01(\t\x12\x19\n\x11serialized_config\x18\x06 \x01(\x0c\x12\x42\n\x06status\x18\x07 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12@\n\x07sources\x18\x08 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x41\n\x08\x66\x65\x61tures\x18\t \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x45\n\x0ctrainingsets\x18\n \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x0b \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x36\n\x04tags\x18\x0c \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\r \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x16\n\nis_deleted\x18\x0e \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\"e\n\x0fProviderRequest\x12>\n\x08provider\x18\x01 \x01(\x0b\x32,.featureform.serving.metadata.proto.Provider\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x8a\x01\n\x0bTrainingSet\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x17\n\x0f\x64\x65\x66\x61ult_variant\x18\x03 \x01(\t\x12\x10\n\x08variants\x18\x04 \x03(\t\"\xfa\x06\n\x12TrainingSetVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\r\n\x05owner\x18\x04 \x01(\t\x12+\n\x07\x63reated\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08provider\x18\x06 \x01(\t\x12\x42\n\x06status\x18\x07 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x41\n\x08\x66\x65\x61tures\x18\x08 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12>\n\x05label\x18\t \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x30\n\x0clast_updated\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08schedule\x18\x0e \x01(\t\x12\x44\n\x0c\x66\x65\x61ture_lags\x18\x0f \x03(\x0b\x32..featureform.serving.metadata.proto.FeatureLag\x12\x36\n\x04tags\x18\x10 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x11 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x13\n\x07task_id\x18\x12 \x01(\tB\x02\x18\x01\x12\x14\n\x0ctask_id_list\x18\x13 \x03(\t\x12^\n\x19resource_snowflake_config\x18\x14 \x01(\x0b\x32;.featureform.serving.metadata.proto.ResourceSnowflakeConfig\x12\x16\n\nis_deleted\x18\x15 \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x16 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\x12\x41\n\x04type\x18\x17 \x01(\x0e\x32\x33.featureform.serving.metadata.proto.TrainingSetType\"\x85\x01\n\x19TrainingSetVariantRequest\x12T\n\x14training_set_variant\x18\x01 \x01(\x0b\x32\x36.featureform.serving.metadata.proto.TrainingSetVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\xb6\x03\n\x06\x45ntity\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x42\n\x06status\x18\x03 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x41\n\x08\x66\x65\x61tures\x18\x04 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x05 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x45\n\x0ctrainingsets\x18\x06 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x36\n\x04tags\x18\x07 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x08 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\"_\n\rEntityRequest\x12:\n\x06\x65ntity\x18\x01 \x01(\x0b\x32*.featureform.serving.metadata.proto.Entity\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\xf1\x02\n\x05Model\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x41\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x04 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x45\n\x0ctrainingsets\x18\x05 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x36\n\x04tags\x18\x06 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x07 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\"\\\n\x0cModelRequest\x12\x38\n\x05model\x18\x01 \x01(\x0b\x32).featureform.serving.metadata.proto.Model\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\xe1\x03\n\x04User\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x41\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x04 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x45\n\x0ctrainingsets\x18\x05 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12@\n\x07sources\x18\x06 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x36\n\x04tags\x18\x08 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\t \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\"Y\n\x0bUserRequest\x12\x36\n\x04user\x18\x01 \x01(\x0b\x32(.featureform.serving.metadata.proto.User\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x85\x01\n\x06Source\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x17\n\x0f\x64\x65\x66\x61ult_variant\x18\x03 \x01(\t\x12\x10\n\x08variants\x18\x04 \x03(\t\"\xbc\x07\n\rSourceVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12L\n\x0etransformation\x18\x0e \x01(\x0b\x32\x32.featureform.serving.metadata.proto.TransformationH\x00\x12\x46\n\x0bprimaryData\x18\x0f \x01(\x0b\x32/.featureform.serving.metadata.proto.PrimaryDataH\x00\x12\r\n\x05owner\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x10\n\x08provider\x18\x06 \x01(\t\x12+\n\x07\x63reated\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x42\n\x06status\x18\x08 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\r\n\x05table\x18\t \x01(\t\x12\x45\n\x0ctrainingsets\x18\n \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x41\n\x08\x66\x65\x61tures\x18\x0b \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x0c \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x30\n\x0clast_updated\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08schedule\x18\x10 \x01(\t\x12\x36\n\x04tags\x18\x11 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x12 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x13\n\x07task_id\x18\x13 \x01(\tB\x02\x18\x01\x12\x33\n\x10max_job_duration\x18\x14 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x14\n\x0ctask_id_list\x18\x15 \x03(\t\x12\x16\n\nis_deleted\x18\x16 \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x17 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\x42\x0c\n\ndefinition\"u\n\x14SourceVariantRequest\x12I\n\x0esource_variant\x18\x01 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.SourceVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"(\n\nSparkParam\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\")\n\x0bWriteOption\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"+\n\rTableProperty\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xe7\x01\n\nSparkFlags\x12\x44\n\x0cspark_params\x18\x01 \x03(\x0b\x32..featureform.serving.metadata.proto.SparkParam\x12\x46\n\rwrite_options\x18\x02 \x03(\x0b\x32/.featureform.serving.metadata.proto.WriteOption\x12K\n\x10table_properties\x18\x03 \x03(\x0b\x32\x31.featureform.serving.metadata.proto.TableProperty\"\x9f\x02\n\x08Location\x12=\n\x05table\x18\x01 \x01(\x0b\x32,.featureform.serving.metadata.proto.SQLTableH\x00\x12G\n\tfilestore\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FileStoreTableH\x00\x12\x43\n\x07\x63\x61talog\x18\x03 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.CatalogTableH\x00\x12:\n\x05kafka\x18\x04 \x01(\x0b\x32).featureform.serving.metadata.proto.KafkaH\x00\x42\n\n\x08location\"\x96\x06\n\x0eTransformation\x12R\n\x11SQLTransformation\x18\x01 \x01(\x0b\x32\x35.featureform.serving.metadata.proto.SQLTransformationH\x00\x12P\n\x10\x44\x46Transformation\x18\x02 \x01(\x0b\x32\x34.featureform.serving.metadata.proto.DFTransformationH\x00\x12M\n\x0fkubernetes_args\x18\x03 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.KubernetesArgsH\x01\x12J\n\rHashPartition\x18\x04 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.HashPartitionH\x02\x12L\n\x0e\x44\x61ilyPartition\x18\x05 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.DailyPartitionH\x02\x12\x43\n\x0bspark_flags\x18\x06 \x01(\x0b\x32..featureform.serving.metadata.proto.SparkFlags\x12=\n\x05table\x18\x07 \x01(\x0b\x32,.featureform.serving.metadata.proto.SQLTableH\x03\x12G\n\tfilestore\x18\x08 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FileStoreTableH\x03\x12\x43\n\x07\x63\x61talog\x18\t \x01(\x0b\x32\x30.featureform.serving.metadata.proto.CatalogTableH\x03\x12:\n\x05kafka\x18\n \x01(\x0b\x32).featureform.serving.metadata.proto.KafkaH\x03\x42\x06\n\x04typeB\x06\n\x04\x61rgsB\x0b\n\tpartitionB\n\n\x08location\"0\n\rHashPartition\x12\x0e\n\x06\x63olumn\x18\x01 \x01(\t\x12\x0f\n\x07\x62uckets\x18\x02 \x01(\x05\" \n\x0e\x44\x61ilyPartition\x12\x0e\n\x06\x63olumn\x18\x01 \x01(\t\"o\n\x17KubernetesResourceSpecs\x12\x13\n\x0b\x63pu_request\x18\x01 \x01(\t\x12\x11\n\tcpu_limit\x18\x02 \x01(\t\x12\x16\n\x0ememory_request\x18\x03 \x01(\t\x12\x14\n\x0cmemory_limit\x18\x04 \x01(\t\"r\n\x0eKubernetesArgs\x12\x14\n\x0c\x64ocker_image\x18\x01 \x01(\t\x12J\n\x05specs\x18\x02 \x01(\x0b\x32;.featureform.serving.metadata.proto.KubernetesResourceSpecs\"\x8a\x03\n\x11SQLTransformation\x12\r\n\x05query\x18\x01 \x01(\t\x12?\n\x06source\x18\x02 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x16\n\x0eis_incremental\x18\x03 \x01(\x08\x12K\n\x12incremental_source\x18\x04 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12^\n\x19resource_snowflake_config\x18\x05 \x01(\x0b\x32;.featureform.serving.metadata.proto.ResourceSnowflakeConfig\x12\x14\n\x0cis_streaming\x18\x06 \x01(\x08\x12J\n\x11streaming_sources\x18\x07 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\"\xd9\x02\n\x10\x44\x46Transformation\x12\r\n\x05query\x18\x01 \x01(\x0c\x12?\n\x06inputs\x18\x02 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x13\n\x0bsource_text\x18\x03 \x01(\t\x12\x16\n\x0eis_incremental\x18\x04 \x01(\x08\x12L\n\x13incremental_sources\x18\x05 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x14\n\x0cis_streaming\x18\x06 \x01(\x08\x12G\n\x0estream_sources\x18\x07 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x1b\n\x13\x63\x61nonical_func_text\x18\x08 \x01(\t\"\xbc\x02\n\x0bPrimaryData\x12=\n\x05table\x18\x01 \x01(\x0b\x32,.featureform.serving.metadata.proto.SQLTableH\x00\x12G\n\tfilestore\x18\x03 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FileStoreTableH\x00\x12\x43\n\x07\x63\x61talog\x18\x04 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.CatalogTableH\x00\x12:\n\x05kafka\x18\x05 \x01(\x0b\x32).featureform.serving.metadata.proto.KafkaH\x00\x12\x18\n\x10timestamp_column\x18\x02 \x01(\tB\n\n\x08location\":\n\x08SQLTable\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tabase\x18\x02 \x01(\t\x12\x0e\n\x06schema\x18\x03 \x01(\t\"\x1e\n\x0e\x46ileStoreTable\x12\x0c\n\x04path\x18\x01 \x01(\t\"\x16\n\x05Kafka\x12\r\n\x05topic\x18\x01 \x01(\t\"E\n\x0c\x43\x61talogTable\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\x12\r\n\x05table\x18\x02 \x01(\t\x12\x14\n\x0ctable_format\x18\x03 \x01(\t\"\x13\n\x04Tags\x12\x0b\n\x03tag\x18\x01 \x03(\t\"+\n\x08Property\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x42\x07\n\x05value\"\xbb\x01\n\nProperties\x12N\n\x08property\x18\x01 \x03(\x0b\x32<.featureform.serving.metadata.proto.Properties.PropertyEntry\x1a]\n\rPropertyEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12;\n\x05value\x18\x02 \x01(\x0b\x32,.featureform.serving.metadata.proto.Property:\x02\x38\x01\"\x7f\n\x17StreamingFeatureVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x0e\n\x06\x65ntity\x18\x03 \x01(\t\x12\r\n\x05value\x18\x04 \x01(\t\x12&\n\x02ts\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"}\n\x15StreamingLabelVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x12\x0e\n\x06\x65ntity\x18\x04 \x01(\t\x12&\n\x02ts\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x8b\x01\n\x17ResourceSnowflakeConfig\x12]\n\x14\x64ynamic_table_config\x18\x01 \x01(\x0b\x32?.featureform.serving.metadata.proto.SnowflakeDynamicTableConfig\x12\x11\n\twarehouse\x18\x02 \x01(\t\"\xbc\x01\n\x1bSnowflakeDynamicTableConfig\x12\x12\n\ntarget_lag\x18\x01 \x01(\t\x12\x45\n\x0crefresh_mode\x18\x02 \x01(\x0e\x32/.featureform.serving.metadata.proto.RefreshMode\x12\x42\n\ninitialize\x18\x03 \x01(\x0e\x32..featureform.serving.metadata.proto.Initialize\"]\n\x16MarkForDeletionRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\"\x19\n\x17MarkForDeletionResponse\"^\n\x17\x46inalizeDeletionRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\"\x1a\n\x18\x46inalizeDeletionResponse\"j\n#GetStagedForDeletionResourceRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\"u\n$GetStagedForDeletionResourceResponse\x12M\n\x10resource_variant\x18\x01 \x01(\x0b\x32\x33.featureform.serving.metadata.proto.ResourceVariant\"[\n\x14PruneResourceRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\"\xb6\x01\n\x15PruneResourceResponse\x12\x12\n\ntarget_lag\x18\x01 \x01(\t\x12\x45\n\x0crefresh_mode\x18\x02 \x01(\x0e\x32/.featureform.serving.metadata.proto.RefreshMode\x12\x42\n\ninitialize\x18\x03 \x01(\x0e\x32..featureform.serving.metadata.proto.Initialize*\xc9\x01\n\x0cResourceType\x12\x0b\n\x07\x46\x45\x41TURE\x10\x00\x12\t\n\x05LABEL\x10\x01\x12\x10\n\x0cTRAINING_SET\x10\x02\x12\n\n\x06SOURCE\x10\x03\x12\x13\n\x0f\x46\x45\x41TURE_VARIANT\x10\x04\x12\x11\n\rLABEL_VARIANT\x10\x05\x12\x18\n\x14TRAINING_SET_VARIANT\x10\x06\x12\x12\n\x0eSOURCE_VARIANT\x10\x07\x12\x0c\n\x08PROVIDER\x10\x08\x12\n\n\x06\x45NTITY\x10\t\x12\t\n\x05MODEL\x10\n\x12\x08\n\x04USER\x10\x0b*F\n\x0f\x43omputationMode\x12\x0f\n\x0bPRECOMPUTED\x10\x00\x12\x13\n\x0f\x43LIENT_COMPUTED\x10\x01\x12\r\n\tSTREAMING\x10\x02*s\n\nScalarType\x12\x08\n\x04NULL\x10\x00\x12\x07\n\x03INT\x10\x01\x12\x0b\n\x07\x46LOAT32\x10\x02\x12\x0b\n\x07\x46LOAT64\x10\x03\x12\n\n\x06STRING\x10\x04\x12\x08\n\x04\x42OOL\x10\x05\x12\t\n\x05INT32\x10\x06\x12\t\n\x05INT64\x10\x07\x12\x0c\n\x08\x44\x41TETIME\x10\x08*%\n\x0bTableFormat\x12\x0b\n\x07ICEBERG\x10\x00\x12\t\n\x05\x44\x45LTA\x10\x01*w\n\x0bRefreshMode\x12\x1c\n\x18REFRESH_MODE_UNSPECIFIED\x10\x00\x12\x15\n\x11REFRESH_MODE_AUTO\x10\x01\x12\x15\n\x11REFRESH_MODE_FULL\x10\x02\x12\x1c\n\x18REFRESH_MODE_INCREMENTAL\x10\x03*^\n\nInitialize\x12\x1a\n\x16INITIALIZE_UNSPECIFIED\x10\x00\x12\x18\n\x14INITIALIZE_ON_CREATE\x10\x01\x12\x1a\n\x16INITIALIZE_ON_SCHEDULE\x10\x02*\x8d\x01\n\x0fTrainingSetType\x12!\n\x1dTRAINING_SET_TYPE_UNSPECIFIED\x10\x00\x12\x1d\n\x19TRAINING_SET_TYPE_DYNAMIC\x10\x01\x12\x1c\n\x18TRAINING_SET_TYPE_STATIC\x10\x02\x12\x1a\n\x16TRAINING_SET_TYPE_VIEW\x10\x03\x32\x82\"\n\x08Metadata\x12h\n\nCreateUser\x12/.featureform.serving.metadata.proto.UserRequest\x1a).featureform.serving.metadata.proto.Empty\x12p\n\x0e\x43reateProvider\x12\x33.featureform.serving.metadata.proto.ProviderRequest\x1a).featureform.serving.metadata.proto.Empty\x12z\n\x13\x43reateSourceVariant\x12\x38.featureform.serving.metadata.proto.SourceVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12l\n\x0c\x43reateEntity\x12\x31.featureform.serving.metadata.proto.EntityRequest\x1a).featureform.serving.metadata.proto.Empty\x12|\n\x14\x43reateFeatureVariant\x12\x39.featureform.serving.metadata.proto.FeatureVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12x\n\x12\x43reateLabelVariant\x12\x37.featureform.serving.metadata.proto.LabelVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12\x84\x01\n\x18\x43reateTrainingSetVariant\x12=.featureform.serving.metadata.proto.TrainingSetVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12j\n\x0b\x43reateModel\x12\x30.featureform.serving.metadata.proto.ModelRequest\x1a).featureform.serving.metadata.proto.Empty\x12}\n\x15RequestScheduleChange\x12\x39.featureform.serving.metadata.proto.ScheduleChangeRequest\x1a).featureform.serving.metadata.proto.Empty\x12i\n\x08GetUsers\x12/.featureform.serving.metadata.proto.NameRequest\x1a(.featureform.serving.metadata.proto.User(\x01\x30\x01\x12o\n\x0bGetFeatures\x12/.featureform.serving.metadata.proto.NameRequest\x1a+.featureform.serving.metadata.proto.Feature(\x01\x30\x01\x12\x84\x01\n\x12GetFeatureVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x32.featureform.serving.metadata.proto.FeatureVariant(\x01\x30\x01\x12k\n\tGetLabels\x12/.featureform.serving.metadata.proto.NameRequest\x1a).featureform.serving.metadata.proto.Label(\x01\x30\x01\x12\x80\x01\n\x10GetLabelVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x30.featureform.serving.metadata.proto.LabelVariant(\x01\x30\x01\x12w\n\x0fGetTrainingSets\x12/.featureform.serving.metadata.proto.NameRequest\x1a/.featureform.serving.metadata.proto.TrainingSet(\x01\x30\x01\x12\x8c\x01\n\x16GetTrainingSetVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x36.featureform.serving.metadata.proto.TrainingSetVariant(\x01\x30\x01\x12m\n\nGetSources\x12/.featureform.serving.metadata.proto.NameRequest\x1a*.featureform.serving.metadata.proto.Source(\x01\x30\x01\x12\x82\x01\n\x11GetSourceVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x31.featureform.serving.metadata.proto.SourceVariant(\x01\x30\x01\x12q\n\x0cGetProviders\x12/.featureform.serving.metadata.proto.NameRequest\x1a,.featureform.serving.metadata.proto.Provider(\x01\x30\x01\x12n\n\x0bGetEntities\x12/.featureform.serving.metadata.proto.NameRequest\x1a*.featureform.serving.metadata.proto.Entity(\x01\x30\x01\x12k\n\tGetModels\x12/.featureform.serving.metadata.proto.NameRequest\x1a).featureform.serving.metadata.proto.Model(\x01\x30\x01\x12\x8a\x01\n\x0fMarkForDeletion\x12:.featureform.serving.metadata.proto.MarkForDeletionRequest\x1a;.featureform.serving.metadata.proto.MarkForDeletionResponse\x12\x8d\x01\n\x10\x46inalizeDeletion\x12;.featureform.serving.metadata.proto.FinalizeDeletionRequest\x1a<.featureform.serving.metadata.proto.FinalizeDeletionResponse\x12\xb1\x01\n\x1cGetStagedForDeletionResource\x12G.featureform.serving.metadata.proto.GetStagedForDeletionResourceRequest\x1aH.featureform.serving.metadata.proto.GetStagedForDeletionResourceResponse\x12\x84\x01\n\rPruneResource\x12\x38.featureform.serving.metadata.proto.PruneResourceRequest\x1a\x39.featureform.serving.metadata.proto.PruneResourceResponse\x12~\n\rGetEquivalent\x12\x38.featureform.serving.metadata.proto.GetEquivalentRequest\x1a\x33.featureform.serving.metadata.proto.ResourceVariant\x12`\n\x03Run\x12..featureform.serving.metadata.proto.RunRequest\x1a).featureform.serving.metadata.proto.Empty\x12n\n\x0cListFeatures\x12/.featureform.serving.metadata.proto.ListRequest\x1a+.featureform.serving.metadata.proto.Feature0\x01\x12j\n\nListLabels\x12/.featureform.serving.metadata.proto.ListRequest\x1a).featureform.serving.metadata.proto.Label0\x01\x12v\n\x10ListTrainingSets\x12/.featureform.serving.metadata.proto.ListRequest\x1a/.featureform.serving.metadata.proto.TrainingSet0\x01\x12l\n\x0bListSources\x12/.featureform.serving.metadata.proto.ListRequest\x1a*.featureform.serving.metadata.proto.Source0\x01\x12h\n\tListUsers\x12/.featureform.serving.metadata.proto.ListRequest\x1a(.featureform.serving.metadata.proto.User0\x01\x12p\n\rListProviders\x12/.featureform.serving.metadata.proto.ListRequest\x1a,.featureform.serving.metadata.proto.Provider0\x01\x12m\n\x0cListEntities\x12/.featureform.serving.metadata.proto.ListRequest\x1a*.featureform.serving.metadata.proto.Entity0\x01\x12j\n\nListModels\x12/.featureform.serving.metadata.proto.ListRequest\x1a).featureform.serving.metadata.proto.Model0\x01\x12t\n\x11SetResourceStatus\x12\x34.featureform.serving.metadata.proto.SetStatusRequest\x1a).featureform.serving.metadata.proto.Empty2\xb5 \n\x03\x41pi\x12h\n\nCreateUser\x12/.featureform.serving.metadata.proto.UserRequest\x1a).featureform.serving.metadata.proto.Empty\x12p\n\x0e\x43reateProvider\x12\x33.featureform.serving.metadata.proto.ProviderRequest\x1a).featureform.serving.metadata.proto.Empty\x12z\n\x13\x43reateSourceVariant\x12\x38.featureform.serving.metadata.proto.SourceVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12l\n\x0c\x43reateEntity\x12\x31.featureform.serving.metadata.proto.EntityRequest\x1a).featureform.serving.metadata.proto.Empty\x12|\n\x14\x43reateFeatureVariant\x12\x39.featureform.serving.metadata.proto.FeatureVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12x\n\x12\x43reateLabelVariant\x12\x37.featureform.serving.metadata.proto.LabelVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12\x84\x01\n\x18\x43reateTrainingSetVariant\x12=.featureform.serving.metadata.proto.TrainingSetVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12j\n\x0b\x43reateModel\x12\x30.featureform.serving.metadata.proto.ModelRequest\x1a).featureform.serving.metadata.proto.Empty\x12}\n\x15RequestScheduleChange\x12\x39.featureform.serving.metadata.proto.ScheduleChangeRequest\x1a).featureform.serving.metadata.proto.Empty\x12i\n\x08GetUsers\x12/.featureform.serving.metadata.proto.NameRequest\x1a(.featureform.serving.metadata.proto.User(\x01\x30\x01\x12o\n\x0bGetFeatures\x12/.featureform.serving.metadata.proto.NameRequest\x1a+.featureform.serving.metadata.proto.Feature(\x01\x30\x01\x12\x84\x01\n\x12GetFeatureVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x32.featureform.serving.metadata.proto.FeatureVariant(\x01\x30\x01\x12k\n\tGetLabels\x12/.featureform.serving.metadata.proto.NameRequest\x1a).featureform.serving.metadata.proto.Label(\x01\x30\x01\x12\x80\x01\n\x10GetLabelVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x30.featureform.serving.metadata.proto.LabelVariant(\x01\x30\x01\x12w\n\x0fGetTrainingSets\x12/.featureform.serving.metadata.proto.NameRequest\x1a/.featureform.serving.metadata.proto.TrainingSet(\x01\x30\x01\x12\x8c\x01\n\x16GetTrainingSetVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x36.featureform.serving.metadata.proto.TrainingSetVariant(\x01\x30\x01\x12m\n\nGetSources\x12/.featureform.serving.metadata.proto.NameRequest\x1a*.featureform.serving.metadata.proto.Source(\x01\x30\x01\x12\x82\x01\n\x11GetSourceVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x31.featureform.serving.metadata.proto.SourceVariant(\x01\x30\x01\x12q\n\x0cGetProviders\x12/.featureform.serving.metadata.proto.NameRequest\x1a,.featureform.serving.metadata.proto.Provider(\x01\x30\x01\x12n\n\x0bGetEntities\x12/.featureform.serving.metadata.proto.NameRequest\x1a*.featureform.serving.metadata.proto.Entity(\x01\x30\x01\x12k\n\tGetModels\x12/.featureform.serving.metadata.proto.NameRequest\x1a).featureform.serving.metadata.proto.Model(\x01\x30\x01\x12\x8a\x01\n\x0fMarkForDeletion\x12:.featureform.serving.metadata.proto.MarkForDeletionRequest\x1a;.featureform.serving.metadata.proto.MarkForDeletionResponse\x12\x84\x01\n\rPruneResource\x12\x38.featureform.serving.metadata.proto.PruneResourceRequest\x1a\x39.featureform.serving.metadata.proto.PruneResourceResponse\x12~\n\rGetEquivalent\x12\x38.featureform.serving.metadata.proto.GetEquivalentRequest\x1a\x33.featureform.serving.metadata.proto.ResourceVariant\x12`\n\x03Run\x12..featureform.serving.metadata.proto.RunRequest\x1a).featureform.serving.metadata.proto.Empty\x12n\n\x0cListFeatures\x12/.featureform.serving.metadata.proto.ListRequest\x1a+.featureform.serving.metadata.proto.Feature0\x01\x12j\n\nListLabels\x12/.featureform.serving.metadata.proto.ListRequest\x1a).featureform.serving.metadata.proto.Label0\x01\x12v\n\x10ListTrainingSets\x12/.featureform.serving.metadata.proto.ListRequest\x1a/.featureform.serving.metadata.proto.TrainingSet0\x01\x12l\n\x0bListSources\x12/.featureform.serving.metadata.proto.ListRequest\x1a*.featureform.serving.metadata.proto.Source0\x01\x12h\n\tListUsers\x12/.featureform.serving.metadata.proto.ListRequest\x1a(.featureform.serving.metadata.proto.User0\x01\x12p\n\rListProviders\x12/.featureform.serving.metadata.proto.ListRequest\x1a,.featureform.serving.metadata.proto.Provider0\x01\x12m\n\x0cListEntities\x12/.featureform.serving.metadata.proto.ListRequest\x1a*.featureform.serving.metadata.proto.Entity0\x01\x12j\n\nListModels\x12/.featureform.serving.metadata.proto.ListRequest\x1a).featureform.serving.metadata.proto.Model0\x01\x12y\n\rWriteFeatures\x12;.featureform.serving.metadata.proto.StreamingFeatureVariant\x1a).featureform.serving.metadata.proto.Empty(\x01\x12u\n\x0bWriteLabels\x12\x39.featureform.serving.metadata.proto.StreamingLabelVariant\x1a).featureform.serving.metadata.proto.Empty(\x01\x42\'Z%github.com/featureform/metadata/protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'featureform.proto.metadata_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z%github.com/featureform/metadata/proto'
  _globals['_FEATUREVARIANT'].fields_by_name['task_id']._options = None
  _globals['_FEATUREVARIANT'].fields_by_name['task_id']._serialized_options = b'\030\001'
  _globals['_FEATUREVARIANT'].fields_by_name['is_deleted']._options = None
  _globals['_FEATUREVARIANT'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_FEATUREVARIANT'].fields_by_name['deleted']._options = None
  _globals['_FEATUREVARIANT'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_LABELVARIANT'].fields_by_name['columns']._options = None
  _globals['_LABELVARIANT'].fields_by_name['columns']._serialized_options = b'\030\001'
  _globals['_LABELVARIANT'].fields_by_name['task_id']._options = None
  _globals['_LABELVARIANT'].fields_by_name['task_id']._serialized_options = b'\030\001'
  _globals['_LABELVARIANT'].fields_by_name['is_deleted']._options = None
  _globals['_LABELVARIANT'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_LABELVARIANT'].fields_by_name['deleted']._options = None
  _globals['_LABELVARIANT'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_PROVIDER'].fields_by_name['is_deleted']._options = None
  _globals['_PROVIDER'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_PROVIDER'].fields_by_name['deleted']._options = None
  _globals['_PROVIDER'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_TRAININGSETVARIANT'].fields_by_name['task_id']._options = None
  _globals['_TRAININGSETVARIANT'].fields_by_name['task_id']._serialized_options = b'\030\001'
  _globals['_TRAININGSETVARIANT'].fields_by_name['is_deleted']._options = None
  _globals['_TRAININGSETVARIANT'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_TRAININGSETVARIANT'].fields_by_name['deleted']._options = None
  _globals['_TRAININGSETVARIANT'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_SOURCEVARIANT'].fields_by_name['task_id']._options = None
  _globals['_SOURCEVARIANT'].fields_by_name['task_id']._serialized_options = b'\030\001'
  _globals['_SOURCEVARIANT'].fields_by_name['is_deleted']._options = None
  _globals['_SOURCEVARIANT'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_SOURCEVARIANT'].fields_by_name['deleted']._options = None
  _globals['_SOURCEVARIANT'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_PROPERTIES_PROPERTYENTRY']._options = None
  _globals['_PROPERTIES_PROPERTYENTRY']._serialized_options = b'8\001'
  _globals['_RESOURCETYPE']._serialized_start=15000
  _globals['_RESOURCETYPE']._serialized_end=15201
  _globals['_COMPUTATIONMODE']._serialized_start=15203
  _globals['_COMPUTATIONMODE']._serialized_end=15273
  _globals['_SCALARTYPE']._serialized_start=15275
  _globals['_SCALARTYPE']._serialized_end=15390
  _globals['_TABLEFORMAT']._serialized_start=15392
  _globals['_TABLEFORMAT']._serialized_end=15429
  _globals['_REFRESHMODE']._serialized_start=15431
  _globals['_REFRESHMODE']._serialized_end=15550
  _globals['_INITIALIZE']._serialized_start=15552
  _globals['_INITIALIZE']._serialized_end=15646
  _globals['_TRAININGSETTYPE']._serialized_start=15649
  _globals['_TRAININGSETTYPE']._serialized_end=15790
  _globals['_PASSTHROUGHAUTHCONFIG']._serialized_start=164
  _globals['_PASSTHROUGHAUTHCONFIG']._serialized_end=187
  _globals['_NAME']._serialized_start=189
  _globals['_NAME']._serialized_end=209
  _globals['_NAMEREQUEST']._serialized_start=211
  _globals['_NAMEREQUEST']._serialized_end=300
  _globals['_ERRORSTATUS']._serialized_start=302
  _globals['_ERRORSTATUS']._serialized_end=385
  _globals['_RESOURCESTATUS']._serialized_start=388
  _globals['_RESOURCESTATUS']._serialized_end=689
  _globals['_RESOURCESTATUS_STATUS']._serialized_start=575
  _globals['_RESOURCESTATUS_STATUS']._serialized_end=689
  _globals['_RESOURCEID']._serialized_start=692
  _globals['_RESOURCEID']._serialized_end=844
  _globals['_SETSTATUSREQUEST']._serialized_start=847
  _globals['_SETSTATUSREQUEST']._serialized_end=1002
  _globals['_SCHEDULECHANGEREQUEST']._serialized_start=1004
  _globals['_SCHEDULECHANGEREQUEST']._serialized_end=1114
  _globals['_NAMEVARIANT']._serialized_start=1116
  _globals['_NAMEVARIANT']._serialized_end=1160
  _globals['_NAMEVARIANTREQUEST']._serialized_start=1162
  _globals['_NAMEVARIANTREQUEST']._serialized_end=1273
  _globals['_EMPTY']._serialized_start=1275
  _globals['_EMPTY']._serialized_end=1282
  _globals['_LISTREQUEST']._serialized_start=1284
  _globals['_LISTREQUEST']._serialized_end=1317
  _globals['_FEATURE']._serialized_start=1320
  _globals['_FEATURE']._serialized_end=1454
  _globals['_COLUMNS']._serialized_start=1456
  _globals['_COLUMNS']._serialized_end=1508
  _globals['_PYTHONFUNCTION']._serialized_start=1510
  _globals['_PYTHONFUNCTION']._serialized_end=1541
  _globals['_STREAM']._serialized_start=1543
  _globals['_STREAM']._serialized_end=1577
  _globals['_RESOURCEVARIANT']._serialized_start=1580
  _globals['_RESOURCEVARIANT']._serialized_end=1928
  _globals['_GETEQUIVALENTREQUEST']._serialized_start=1930
  _globals['_GETEQUIVALENTREQUEST']._serialized_end=2042
  _globals['_RUNREQUEST']._serialized_start=2044
  _globals['_RUNREQUEST']._serialized_end=2147
  _globals['_FEATUREVARIANT']._serialized_start=2150
  _globals['_FEATUREVARIANT']._serialized_end=3455
  _globals['_FEATUREVARIANTREQUEST']._serialized_start=3457
  _globals['_FEATUREVARIANTREQUEST']._serialized_end=3577
  _globals['_VALUETYPE']._serialized_start=3580
  _globals['_VALUETYPE']._serialized_end=3731
  _globals['_VECTORTYPE']._serialized_start=3733
  _globals['_VECTORTYPE']._serialized_end=3850
  _globals['_FEATUREPARAMETERS']._serialized_start=3853
  _globals['_FEATUREPARAMETERS']._serialized_end=4060
  _globals['_PRECOMPUTEDFEATUREPARAMETERS']._serialized_start=4062
  _globals['_PRECOMPUTEDFEATUREPARAMETERS']._serialized_end=4092
  _globals['_ONDEMANDFEATUREPARAMETERS']._serialized_start=4094
  _globals['_ONDEMANDFEATUREPARAMETERS']._serialized_end=4141
  _globals['_FEATURELAG']._serialized_start=4143
  _globals['_FEATURELAG']._serialized_end=4243
  _globals['_LABEL']._serialized_start=4246
  _globals['_LABEL']._serialized_end=4378
  _globals['_LABELVARIANT']._serialized_start=4381
  _globals['_LABELVARIANT']._serialized_end=5363
  _globals['_ENTITYMAPPINGS']._serialized_start=5366
  _globals['_ENTITYMAPPINGS']._serialized_end=5499
  _globals['_ENTITYMAPPING']._serialized_start=5501
  _globals['_ENTITYMAPPING']._serialized_end=5553
  _globals['_LABELVARIANTREQUEST']._serialized_start=5555
  _globals['_LABELVARIANTREQUEST']._serialized_end=5669
  _globals['_PROVIDER']._serialized_start=5672
  _globals['_PROVIDER']._serialized_end=6324
  _globals['_PROVIDERREQUEST']._serialized_start=6326
  _globals['_PROVIDERREQUEST']._serialized_end=6427
  _globals['_TRAININGSET']._serialized_start=6430
  _globals['_TRAININGSET']._serialized_end=6568
  _globals['_TRAININGSETVARIANT']._serialized_start=6571
  _globals['_TRAININGSETVARIANT']._serialized_end=7461
  _globals['_TRAININGSETVARIANTREQUEST']._serialized_start=7464
  _globals['_TRAININGSETVARIANTREQUEST']._serialized_end=7597
  _globals['_ENTITY']._serialized_start=7600
  _globals['_ENTITY']._serialized_end=8038
  _globals['_ENTITYREQUEST']._serialized_start=8040
  _globals['_ENTITYREQUEST']._serialized_end=8135
  _globals['_MODEL']._serialized_start=8138
  _globals['_MODEL']._serialized_end=8507
  _globals['_MODELREQUEST']._serialized_start=8509
  _globals['_MODELREQUEST']._serialized_end=8601
  _globals['_USER']._serialized_start=8604
  _globals['_USER']._serialized_end=9085
  _globals['_USERREQUEST']._serialized_start=9087
  _globals['_USERREQUEST']._serialized_end=9176
  _globals['_SOURCE']._serialized_start=9179
  _globals['_SOURCE']._serialized_end=9312
  _globals['_SOURCEVARIANT']._serialized_start=9315
  _globals['_SOURCEVARIANT']._serialized_end=10271
  _globals['_SOURCEVARIANTREQUEST']._serialized_start=10273
  _globals['_SOURCEVARIANTREQUEST']._serialized_end=10390
  _globals['_SPARKPARAM']._serialized_start=10392
  _globals['_SPARKPARAM']._serialized_end=10432
  _globals['_WRITEOPTION']._serialized_start=10434
  _globals['_WRITEOPTION']._serialized_end=10475
  _globals['_TABLEPROPERTY']._serialized_start=10477
  _globals['_TABLEPROPERTY']._serialized_end=10520
  _globals['_SPARKFLAGS']._serialized_start=10523
  _globals['_SPARKFLAGS']._serialized_end=10754
  _globals['_LOCATION']._serialized_start=10757
  _globals['_LOCATION']._serialized_end=11044
  _globals['_TRANSFORMATION']._serialized_start=11047
  _globals['_TRANSFORMATION']._serialized_end=11837
  _globals['_HASHPARTITION']._serialized_start=11839
  _globals['_HASHPARTITION']._serialized_end=11887
  _globals['_DAILYPARTITION']._serialized_start=11889
  _globals['_DAILYPARTITION']._serialized_end=11921
  _globals['_KUBERNETESRESOURCESPECS']._serialized_start=11923
  _globals['_KUBERNETESRESOURCESPECS']._serialized_end=12034
  _globals['_KUBERNETESARGS']._serialized_start=12036
  _globals['_KUBERNETESARGS']._serialized_end=12150
  _globals['_SQLTRANSFORMATION']._serialized_start=12153
  _globals['_SQLTRANSFORMATION']._serialized_end=12547
  _globals['_DFTRANSFORMATION']._serialized_start=12550
  _globals['_DFTRANSFORMATION']._serialized_end=12895
  _globals['_PRIMARYDATA']._serialized_start=12898
  _globals['_PRIMARYDATA']._serialized_end=13214
  _globals['_SQLTABLE']._serialized_start=13216
  _globals['_SQLTABLE']._serialized_end=13274
  _globals['_FILESTORETABLE']._serialized_start=13276
  _globals['_FILESTORETABLE']._serialized_end=13306
  _globals['_KAFKA']._serialized_start=13308
  _globals['_KAFKA']._serialized_end=13330
  _globals['_CATALOGTABLE']._serialized_start=13332
  _globals['_CATALOGTABLE']._serialized_end=13401
  _globals['_TAGS']._serialized_start=13403
  _globals['_TAGS']._serialized_end=13422
  _globals['_PROPERTY']._serialized_start=13424
  _globals['_PROPERTY']._serialized_end=13467
  _globals['_PROPERTIES']._serialized_start=13470
  _globals['_PROPERTIES']._serialized_end=13657
  _globals['_PROPERTIES_PROPERTYENTRY']._serialized_start=13564
  _globals['_PROPERTIES_PROPERTYENTRY']._serialized_end=13657
  _globals['_STREAMINGFEATUREVARIANT']._serialized_start=13659
  _globals['_STREAMINGFEATUREVARIANT']._serialized_end=13786
  _globals['_STREAMINGLABELVARIANT']._serialized_start=13788
  _globals['_STREAMINGLABELVARIANT']._serialized_end=13913
  _globals['_RESOURCESNOWFLAKECONFIG']._serialized_start=13916
  _globals['_RESOURCESNOWFLAKECONFIG']._serialized_end=14055
  _globals['_SNOWFLAKEDYNAMICTABLECONFIG']._serialized_start=14058
  _globals['_SNOWFLAKEDYNAMICTABLECONFIG']._serialized_end=14246
  _globals['_MARKFORDELETIONREQUEST']._serialized_start=14248
  _globals['_MARKFORDELETIONREQUEST']._serialized_end=14341
  _globals['_MARKFORDELETIONRESPONSE']._serialized_start=14343
  _globals['_MARKFORDELETIONRESPONSE']._serialized_end=14368
  _globals['_FINALIZEDELETIONREQUEST']._serialized_start=14370
  _globals['_FINALIZEDELETIONREQUEST']._serialized_end=14464
  _globals['_FINALIZEDELETIONRESPONSE']._serialized_start=14466
  _globals['_FINALIZEDELETIONRESPONSE']._serialized_end=14492
  _globals['_GETSTAGEDFORDELETIONRESOURCEREQUEST']._serialized_start=14494
  _globals['_GETSTAGEDFORDELETIONRESOURCEREQUEST']._serialized_end=14600
  _globals['_GETSTAGEDFORDELETIONRESOURCERESPONSE']._serialized_start=14602
  _globals['_GETSTAGEDFORDELETIONRESOURCERESPONSE']._serialized_end=14719
  _globals['_PRUNERESOURCEREQUEST']._serialized_start=14721
  _globals['_PRUNERESOURCEREQUEST']._serialized_end=14812
  _globals['_PRUNERESOURCERESPONSE']._serialized_start=14815
  _globals['_PRUNERESOURCERESPONSE']._serialized_end=14997
  _globals['_METADATA']._serialized_start=15793
  _globals['_METADATA']._serialized_end=20147
  _globals['_API']._serialized_start=20150
  _globals['_API']._serialized_end=24299
# @@protoc_insertion_point(module_scope)



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

from featureform.proto import metadata_pb2
from .format import *

cutoff_length = 60


def list_name(stub, resource_type):
    stub_list_functions = {
        "model": stub.ListModels,
    }
    format_rows("NAME")
    res = sorted(
        [
            received
            for received in stub_list_functions[resource_type](
                metadata_pb2.ListRequest()
            )
        ],
        key=lambda x: x.name,
    )
    for f in res:
        format_rows(f.name)
    return res


def list_name_status(stub, resource_type):
    stub_list_functions = {"entity": stub.ListEntities, "user": stub.ListUsers}
    format_rows("NAME", "STATUS")
    res = sorted(
        [
            received
            for received in stub_list_functions[resource_type](
                metadata_pb2.ListRequest()
            )
        ],
        key=lambda x: x.name,
    )
    for f in res:
        format_rows(f.name, f.status.Status._enum_type.values[f.status.status].name)
    return res


def list_name_status_desc(stub, resource_type):
    stub_list_functions = {"model": stub.ListModels, "provider": stub.ListProviders}
    format_rows("NAME", "STATUS", "DESCRIPTION")
    res = sorted(
        [
            received
            for received in stub_list_functions[resource_type](
                metadata_pb2.ListRequest()
            )
        ],
        key=lambda x: x.name,
    )
    for f in res:
        format_rows(
            f.name,
            f.status.Status._enum_type.values[f.status.status].name,
            f.description[:cutoff_length],
        )
    return res


def list_name_variant_status(stub, resource_type):
    stub_list_functions = {
        "feature": [stub.ListFeatures, stub.GetFeatureVariants],
        "label": [stub.ListLabels, stub.GetLabelVariants],
    }

    format_rows("NAME", "VARIANT", "STATUS")
    res = sorted(
        [
            received
            for received in stub_list_functions[resource_type][0](
                metadata_pb2.ListRequest()
            )
        ],
        key=lambda x: x.name,
    )
    for f in res:
        for v in f.variants:

            searchNameVariant = metadata_pb2.NameVariant(name=f.name, variant=v)
            req = metadata_pb2.NameVariantRequest(name_variant=searchNameVariant)
            for x in stub_list_functions[resource_type][1](iter([req])):
                if x.variant == f.default_variant:
                    format_rows(
                        f.name,
                        f"{f.default_variant} (default)",
                        x.status.Status._enum_type.values[x.status.status].name,
                    )
                else:
                    format_rows(
                        x.name,
                        x.variant,
                        x.status.Status._enum_type.values[x.status.status].name,
                    )
    return res


def list_name_variant_status_desc(stub, resource_type):
    stub_list_functions = {
        "source": [stub.ListSources, stub.GetSourceVariants],
        "training-set": [stub.ListTrainingSets, stub.GetTrainingSetVariants],
        "trainingset": [stub.ListTrainingSets, stub.GetTrainingSetVariants],
    }

    format_rows("NAME", "VARIANT", "STATUS", "DESCRIPTION")
    res = sorted(
        [
            received
            for received in stub_list_functions[resource_type][0](
                metadata_pb2.ListRequest()
            )
        ],
        key=lambda x: x.name,
    )
    for f in res:
        for v in f.variants:
            searchNameVariant = metadata_pb2.NameVariant(name=f.name, variant=v)
            req = metadata_pb2.NameVariantRequest(name_variant=searchNameVariant)
            for x in stub_list_functions[resource_type][1](iter([req])):
                if x.variant == f.default_variant:
                    format_rows(
                        f.name,
                        f"{f.default_variant} (default)",
                        x.status.Status._enum_type.values[x.status.status].name,
                        x.description,
                    )
                else:
                    format_rows(
                        x.name,
                        x.variant,
                        x.status.Status._enum_type.values[x.status.status].name,
                        x.description,
                    )
    return res



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

import pytest

import featureform as ff
from featureform import get_random_name
from featureform.register import Registrar, DFTransformationDecorator, ResourceRegistrar
from featureform.resources import Entity, FeatureVariant, LabelVariant


@pytest.mark.parametrize(
    "provider_source_fxt,is_local,is_insecure",
    [
        pytest.param(
            "hosted_sql_provider_and_source", False, False, marks=pytest.mark.hosted
        ),
        pytest.param(
            "hosted_sql_provider_and_source", False, True, marks=pytest.mark.docker
        ),
    ],
)
def test_class_api_syntax(provider_source_fxt, is_local, is_insecure, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    provider, source, inference_store = request.getfixturevalue(provider_source_fxt)(
        custom_marks
    )

    arrange_resources(provider, source, inference_store, is_local, is_insecure)
    original_syntax_state = list(
        filter(
            lambda r: isinstance(r, (Entity, FeatureVariant, LabelVariant)),
            ff.state().sorted_list(),
        )
    )

    ff.clear_state()

    arrange_resources(
        provider, source, inference_store, is_local, is_insecure, is_class_api=True
    )
    class_syntax_state = list(
        filter(
            lambda r: isinstance(r, (Entity, FeatureVariant, LabelVariant)),
            ff.state().sorted_list(),
        )
    )

    # since the api's can have source objects rather than the name variants, we're going to get them in the same state
    _update_sources_to_name_variant(original_syntax_state)
    _update_sources_to_name_variant(class_syntax_state)

    assert original_syntax_state == class_syntax_state


def _update_sources_to_name_variant(resources):
    for r in resources:
        if isinstance(r, (FeatureVariant, LabelVariant)):
            if hasattr(r.source, "name_variant"):
                r.source = r.source.name_variant()


@pytest.mark.parametrize(
    "provider_source_fxt,is_local",
    [
        pytest.param("hosted_sql_provider_and_source", False, marks=pytest.mark.hosted),
        pytest.param("hosted_sql_provider_and_source", False, marks=pytest.mark.docker),
    ],
)
def test_variants_naming_consistency(provider_source_fxt, is_local, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    _, source, _ = request.getfixturevalue(provider_source_fxt)(custom_marks)
    label_column = "IsFraud" if is_local else "isfraud"
    label_entity_column = "CustomerID" if is_local else "customerid"

    with pytest.raises(ValueError):
        ff.Variants(
            {
                "quickstart": ff.Label(
                    source[[label_entity_column, label_column]],
                    description="Whether a user's transaction is fraudulent.",
                    variant="quickstart_v2",
                    type=ff.Bool,
                ),
                "quickstart_v2": ff.Label(
                    source[[label_entity_column, label_column]],
                    description="Whether a user's transaction is fraudulent.",
                    variant="quickstart",
                    type=ff.Bool,
                ),
            }
        )


@pytest.mark.local
def test_subscriptable_transformation_decorator_method_call():
    variant = get_random_name()
    df_transformation_decorator = DFTransformationDecorator(
        registrar=Registrar(),
        provider="test_provider",
        owner="test_owner",
        tags=[],
        properties={},
        variant=variant,
    )

    def test_function():
        pass

    df_transformation_decorator(test_function)

    name_variant = df_transformation_decorator.name_variant()
    column_resource = df_transformation_decorator.register_resources(
        entity="user",
        entity_column="user_id",
        owner="test_owner",
        inference_store="test_store",
        features=[
            {
                "name": "avg_transactions",
                "variant": "quickstart",
                "column": "TransactionAmount",
                "type": "float32",
            }
        ],
    )

    assert name_variant == ("test_function", variant) and isinstance(
        column_resource, ResourceRegistrar
    )


@pytest.mark.parametrize(
    "provider_source_fxt,is_local",
    [
        pytest.param("hosted_sql_provider_and_source", False, marks=pytest.mark.hosted),
        pytest.param("hosted_sql_provider_and_source", False, marks=pytest.mark.docker),
    ],
)
def test_indexing_with_fewer_than_two_columns(provider_source_fxt, is_local, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    _, source, _ = request.getfixturevalue(provider_source_fxt)(custom_marks)
    label_entity_column = "CustomerID" if is_local else "customerid"

    with pytest.raises(Exception, match="Expected 2 columns"):
        source[[label_entity_column]]


@pytest.mark.parametrize(
    "provider_source_fxt,is_local",
    [
        pytest.param("hosted_sql_provider_and_source", False, marks=pytest.mark.hosted),
        pytest.param("hosted_sql_provider_and_source", False, marks=pytest.mark.docker),
    ],
)
def test_indexing_with_more_than_three_columns(provider_source_fxt, is_local, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    _, source, _ = request.getfixturevalue(provider_source_fxt)(custom_marks)
    label_entity_column = "CustomerID" if is_local else "customerid"
    label_timestamp_column = "Timestamp" if is_local else "timestamp"

    with pytest.raises(Exception, match="Found unrecognized columns"):
        source[
            [
                label_entity_column,
                label_entity_column,
                label_timestamp_column,
                "unknown_column",
            ]
        ]


@pytest.mark.parametrize(
    "provider_source_fxt,is_local",
    [
        pytest.param("hosted_sql_provider_and_source", False, marks=pytest.mark.hosted),
        pytest.param("hosted_sql_provider_and_source", False, marks=pytest.mark.docker),
    ],
)
def test_specifying_timestamp_column_twice(provider_source_fxt, is_local, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    _, source, _ = request.getfixturevalue(provider_source_fxt)(custom_marks)
    label_column = "IsFraud" if is_local else "isfraud"
    label_entity_column = "CustomerID" if is_local else "customerid"
    timestamp_column = "Timestamp" if is_local else "timestamp"

    with pytest.raises(Exception, match="Timestamp column specified twice"):
        ff.Label(
            source[[label_entity_column, label_column, timestamp_column]],
            description="Whether a user's transaction is fraudulent.",
            variant="quickstart",
            type=ff.Bool,
            timestamp_column=timestamp_column,
        )


@pytest.fixture(autouse=True)
def before_and_after_each(setup_teardown):
    setup_teardown()
    yield
    setup_teardown()


def arrange_resources(
    provider, source, online_store, is_local, is_insecure, is_class_api=False
):
    if is_local:

        @provider.df_transformation(
            variant="quickstart", inputs=[("transactions", "quickstart")]
        )
        def average_user_transaction(transactions):
            return transactions.groupby("CustomerID")["TransactionAmount"].mean()

    else:

        @provider.sql_transformation(variant="quickstart")
        def average_user_transaction():
            return "SELECT customerid as user_id, avg(transactionamount) as avg_transaction_amt from {{transactions.quickstart}} GROUP BY user_id"

    inference_store = provider if is_local else online_store
    feature_column = "TransactionAmount" if is_local else "avg_transaction_amt"
    label_column = "IsFraud" if is_local else "isfraud"
    feature_entity_column = "CustomerID" if is_local else "user_id"
    label_entity_column = "CustomerID" if is_local else "customerid"
    timestamp_column = "Timestamp" if is_local else "timestamp"

    if is_class_api:

        @ff.entity
        class User:
            fraudulent = ff.Label(
                source[[label_entity_column, label_column, timestamp_column]],
                description="Whether a user's transaction is fraudulent.",
                variant="quickstart",
                type=ff.Bool,
            )
            avg_transactions = ff.Variants(
                {
                    "quickstart": ff.Feature(
                        average_user_transaction[
                            [feature_entity_column, feature_column]
                        ],
                        variant="quickstart",
                        type=ff.Float32,
                        inference_store=inference_store,
                        owner="test_user",
                        tags=["quickstart"],
                    ),
                    "quickstart_v2": ff.Feature(
                        average_user_transaction[
                            [feature_entity_column, feature_column]
                        ],
                        variant="quickstart_v2",
                        type=ff.Float32,
                        inference_store=inference_store,
                        owner="test_user",
                        properties={"version": "2.0"},
                    ),
                }
            )

    else:
        user = ff.register_entity("user")

        average_user_transaction.register_resources(
            entity=user,
            entity_column=feature_entity_column,
            inference_store=inference_store,
            features=[
                {
                    "name": "avg_transactions",
                    "variant": "quickstart",
                    "column": feature_column,
                    "type": "float32",
                    "tags": ["quickstart"],
                    "owner": "test_user",
                },
                {
                    "name": "avg_transactions",
                    "variant": "quickstart_v2",
                    "column": feature_column,
                    "type": "float32",
                    "properties": {"version": "2.0"},
                    "owner": "test_user",
                },
            ],
        )

        source.register_resources(
            entity=user,
            entity_column=label_entity_column,
            labels=[
                {
                    "name": "fraudulent",
                    "variant": "quickstart",
                    "column": label_column,
                    "type": "bool",
                    "description": "Whether a user's transaction is fraudulent.",
                },
            ],
            timestamp_column=timestamp_column,
        )



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

import featureform as ff
import inspect
from typing import Union, Optional
from featureform.register import (
    SourceRegistrar,
    SubscriptableTransformation,
)


def test_client_has_columns_method():
    client = ff.Client(host="localhost:7878", insecure=True, dry_run=True)
    assert hasattr(client, "columns")


def test_columns_method_signature():
    sig = inspect.signature(ff.Client.columns)

    params = sig.parameters

    assert (
        params["source"].annotation
        == Union[SourceRegistrar, SubscriptableTransformation, str]
    )
    assert params["variant"].annotation == Optional[str]
    assert params["variant"].default == None



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

import requests
from .format import *


def search(phrase, host):
    response = requests.get(f"http://{host}/data/search?q={phrase}")

    if response.status_code != 200:
        print(
            f"Search request for {phrase} resulted in HTTP status {response.status_code}"
        )
        return

    results = response.json()

    if len(results) == 0:
        print(f"Search phrase {phrase} returned no results.")
    else:
        format_rows("NAME", "VARIANT", "TYPE")
        for r in results:
            format_rows(r["Name"], r["Variant"], r["Type"])

    return results



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

import featureform as ff
from featureform.resources import ResourceRedefinedError
import json
import pytest
from types import SimpleNamespace


# TODO: Make this test pass
@pytest.mark.parametrize(
    "provider_source_fxt,is_local,is_insecure",
    [
        pytest.param("local_provider_source", True, True, marks=pytest.mark.local),
        pytest.param(
            "hosted_sql_provider_and_source", False, False, marks=pytest.mark.hosted
        ),
        pytest.param(
            "hosted_sql_provider_and_source", False, True, marks=pytest.mark.docker
        ),
    ],
)
def xtest_valid_provider_update(provider_source_fxt, is_local, is_insecure, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    provider, source, inference_store = request.getfixturevalue(provider_source_fxt)(
        custom_marks
    )

    # Arranges the resources context following the Quickstart pattern
    resource_client, postgres_name, redis_name = arrange_resources(
        provider, source, inference_store, is_local, is_insecure
    )

    # Must clear global state to simulate two separate calls to apply;
    # otherwise, a ResourceRedefinedError is thrown
    ff.clear_state()

    # DESCRIPTION
    postgres_description = "This is an updated description for Provider A"
    redis_description = "This is an updated description for Provider B"
    # USERNAME
    postgres_username = "username_a_updated"
    # PASSWORD
    postgres_password = "password_a_updated"
    redis_password = "password_b_updated"

    if is_local:
        postgres_host = "0.0.0.0"
        redis_host = "0.0.0.0"
    else:
        # The host name for postgres is different between Docker and Minikube
        postgres_host = (
            "host.docker.internal"
            if "docker" in custom_marks
            else "quickstart-postgres"
        )
        redis_host = (
            "host.docker.internal" if "docker" in custom_marks else "quickstart-redis"
        )

    postgres = ff.register_postgres(
        name=postgres_name,
        host=postgres_host,
        port="5432",
        database="postgres",
        user=postgres_username,
        password=postgres_password,
        description=postgres_description,
    )

    redis = ff.register_redis(
        name=redis_name,
        host=redis_host,
        port=6379,
        password=redis_password,
        description=redis_description,
    )

    resource_client.apply()

    updated_postgres = resource_client.get_provider(postgres_name, is_local)
    updated_redis = resource_client.get_provider(redis_name, is_local)
    print(updated_postgres, updated_redis)
    postgres_config, redis_config = get_postgres_redis_configs(
        updated_postgres, updated_redis, is_local
    )

    postgres_updates = [
        # updated_postgres["description"] == postgres_description if is_local else updated_postgres.description == postgres_description,
        postgres_config.Username == postgres_username,
        postgres_config.Password == postgres_password,
    ]

    redis_updates = [
        # updated_redis["description"] == redis_description if is_local else updated_redis.description == redis_description,
        redis_config.Password
        == redis_password,
    ]

    print(postgres_updates, redis_updates)

    assert all(postgres_updates) and all(redis_updates)


# TODO: Make this test pass
@pytest.mark.parametrize(
    "provider_source_fxt,is_local,is_insecure",
    [
        pytest.param("local_provider_source", True, True, marks=pytest.mark.local),
        pytest.param(
            "hosted_sql_provider_and_source", False, False, marks=pytest.mark.hosted
        ),
        pytest.param(
            "hosted_sql_provider_and_source", False, True, marks=pytest.mark.docker
        ),
    ],
)
def xtest_invalid_provider_update(
    provider_source_fxt, is_local, is_insecure, request, capsys
):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    provider, source, inference_store = request.getfixturevalue(provider_source_fxt)(
        custom_marks
    )

    # Arranges the resources context following the Quickstart pattern
    resource_client, postgres_name, redis_name = arrange_resources(
        provider, source, inference_store, is_local, is_insecure
    )

    # Must clear global state to simulate two separate calls to apply;
    # otherwise, a ResourceRedefinedError is thrown
    ff.clear_state()

    updated_postgres_host = "updated-quickstart-postgres"
    updated_postgres_database = "updated-postgres"

    postgres = ff.register_postgres(
        name=postgres_name,
        host=updated_postgres_host,
        database=updated_postgres_database,
        port="5432",
        user="postgres",
        password="password",
        description="A Postgres deployment we created for the Featureform quickstart",
    )

    resource_client.apply()
    out, err = capsys.readouterr()
    postgres_logs = [ln for ln in out.split("\n")]

    ff.clear_state()

    updated_redis_host = "updated-quickstart-redis"
    updated_redis_port = 6380

    redis = ff.register_redis(
        name=redis_name,
        host=updated_redis_host,
        port=updated_redis_port,
    )

    resource_client.apply()
    out, err = capsys.readouterr()
    redis_logs = [ln for ln in out.split("\n")]

    updated_postgres = resource_client.get_provider(postgres_name, is_local)
    updated_redis = resource_client.get_provider(redis_name, is_local)

    postgres_config, redis_config = get_postgres_redis_configs(
        updated_postgres, updated_redis, is_local
    )

    no_updates = [
        postgres_config.Host != updated_postgres_host,
        postgres_config.Database != updated_postgres_database,
        redis_config.Addr != f"{updated_redis_host}:{updated_redis_port}",
    ]

    assert (
        "postgres-quickstart already exists." in postgres_logs
        and "redis-quickstart already exists." in redis_logs
        and all(no_updates)
    )


@pytest.fixture(autouse=True)
def before_and_after_each(setup_teardown):
    setup_teardown()
    yield
    setup_teardown()


def arrange_resources(provider, source, online_store, is_local, is_insecure):
    if is_local:
        postgres_name = "postgres-quickstart"
        redis_name = "redis-quickstart"

        postgres = ff.register_postgres(
            name="postgres-quickstart",
            host="0.0.0.0",
            port="5432",
            user="postgres",
            password="password",
            database="postgres",
            description="A Postgres deployment we created for the Featureform quickstart",
        )

        redis = ff.register_redis(
            name="redis-quickstart",
            host="0.0.0.0",
            port=6379,
        )
    else:
        postgres_name = provider._OfflineProvider__provider.name
        redis_name = online_store._OnlineProvider__provider.name

    resource_client = ff.ResourceClient(local=is_local, insecure=is_insecure)
    resource_client.apply()

    return (resource_client, postgres_name, redis_name)


def get_postgres_redis_configs(postgres_resource, redis_resource, is_local):
    if is_local:
        postgres_config_json = postgres_resource["serializedConfig"]
        redis_config_json = redis_resource["serializedConfig"]
    else:
        postgres_config_json = postgres_resource.serialized_config.decode("UTF-8")
        redis_config_json = redis_resource.serialized_config.decode("UTF-8")

    postgres_config = json.loads(
        postgres_config_json, object_hook=lambda d: SimpleNamespace(**d)
    )
    redis_config = json.loads(
        redis_config_json, object_hook=lambda d: SimpleNamespace(**d)
    )

    return (postgres_config, redis_config)



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

from featureform.parse import add_variant_to_name


def test_replace_name_with_default():
    text = "something something {{ name }} something else"
    result = add_variant_to_name(text, "default")
    assert result == "something something {{ name.default }} something else"


def test_replace_name_with_custom_default():
    text = "something something {{ name }} something else"
    result = add_variant_to_name(text, "custom_default")
    assert result == "something something {{ name.custom_default }} something else"


def test_replace_multiple_names():
    text = "something {{ name1 }} something {{ name2 }} something else"
    result = add_variant_to_name(text, "custom_default")
    assert (
        result
        == "something {{ name1.custom_default }} something {{ name2.custom_default }} something else"
    )


def test_replace_with_period():
    text = "something {{ name.variant }} something"
    result = add_variant_to_name(text, "custom_default")
    assert result == "something {{ name.variant }} something"


def test_replace_with_underscores():
    text = "something {{ name_my_name }} something"
    result = add_variant_to_name(text, "default")
    assert result == "something {{ name_my_name.default }} something"


def test_replace_no_space():
    text = "something {{name}} something"
    result = add_variant_to_name(text, "default")
    assert result == "something {{ name.default }} something"


def test_replace_extra_space():
    text = "something {{   name    }} something"
    result = add_variant_to_name(text, "default")
    assert result == "something {{ name.default }} something"



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

import pytest

from featureform.register import Registrar, ColumnMapping
from featureform.resources import SQLTable


def test_set_run():
    registrar = Registrar()
    registrar.set_run("test")
    assert registrar.get_run() == "test"


@pytest.mark.parametrize("set_variant", [False, True])
def test_register_columns(set_variant):
    registrar = Registrar()
    if set_variant:
        registrar.set_run("test")
    run = registrar.get_run()
    registrar.register_column_resources(
        ("name", ""),
        entity="",
        entity_column="",
        owner="default_user",
        features=[
            ColumnMapping(name="name", type="int", source="source", column="column")
        ],
        inference_store="redis",
    )
    resource = registrar.state().sorted_list()[0]
    assert resource.variant == run
    assert resource.source[1] == run


@pytest.mark.parametrize("set_variant", [False, True])
def test_register_primary(set_variant):
    registrar = Registrar()
    if set_variant:
        registrar.set_run("test")
    run = registrar.get_run()
    registrar.register_primary_data(
        name="test",
        owner="default_user",
        location=SQLTable(name="table"),
        provider="",
        tags=[],
        properties={},
    )
    resource = registrar.state().sorted_list()[0]
    assert resource.variant == run


@pytest.mark.parametrize("set_variant", [False, True])
def test_register_sql(set_variant):
    registrar = Registrar()
    if set_variant:
        registrar.set_run("test")
    run = registrar.get_run()

    @registrar.sql_transformation(
        name="test",
        owner="default_user",
        provider="",
    )
    def query():
        return "SELECT * FROM {{ table }}"

    resource = registrar.state().sorted_list()[0]
    assert resource.definition.query == "SELECT * FROM {{ table." + run + " }}"
    assert resource.variant == run


@pytest.mark.parametrize("set_variant", [False, True])
def test_register_df(set_variant):
    registrar = Registrar()
    if set_variant:
        registrar.set_run("test")
    run = registrar.get_run()

    @registrar.df_transformation(
        name="test",
        owner="default_user",
        provider="",
        inputs=[("name", ""), "name2"],
        tags=[],
        properties={},
    )
    def some_function(x):
        return x

    resource = registrar.state().sorted_list()[0]
    for input in resource.definition.inputs:
        assert input[1] == run
    assert resource.variant == run


@pytest.mark.parametrize("set_variant", [False, True])
def test_register_trainingset(set_variant):
    registrar = Registrar()
    if set_variant:
        registrar.set_run("test")
    run = registrar.get_run()
    registrar.register_training_set(
        name="test",
        owner="default_user",
        features=[("feature", "")],
        label=("label", ""),
        tags=[],
        properties={},
    )
    resource = registrar.state().sorted_list()[0]
    assert resource.variant == run
    for feature in resource.features:
        assert feature[1] == run
    assert resource.label[1] == run


@pytest.mark.parametrize("set_variant", [False, True])
def test_register_trainingset_string(set_variant):
    registrar = Registrar()
    if set_variant:
        registrar.set_run("test")
    run = registrar.get_run()
    registrar.register_training_set(
        name="test",
        owner="default_user",
        features=["feature"],
        label="label",
        tags=[],
        properties={},
    )
    resource = registrar.state().sorted_list()[0]
    assert resource.variant == run
    for feature in resource.features:
        assert feature[1] == run
    assert resource.label[1] == run


def test_register_columns_with_variant():
    variant = "test"
    registrar = Registrar()
    run = registrar.get_run()
    registrar.register_column_resources(
        ("name", ""),
        entity="",
        entity_column="",
        owner="default_user",
        features=[
            ColumnMapping(
                name="name",
                variant=variant,
                type="int",
                source="source",
                column="column",
            )
        ],
        inference_store="redis",
    )
    resource = registrar.state().sorted_list()[0]
    assert resource.variant == variant
    assert resource.source[1] == run


def test_register_primary_with_variant():
    variant = "test"
    registrar = Registrar()
    registrar.register_primary_data(
        name="test",
        variant=variant,
        owner="default_user",
        location=SQLTable(name="table"),
        provider="",
        tags=[],
        properties={},
    )
    resource = registrar.state().sorted_list()[0]
    assert resource.variant == variant


def test_register_sql_with_variant():
    variant = "test"
    registrar = Registrar()
    run = registrar.get_run()

    @registrar.sql_transformation(
        name="test",
        variant=variant,
        owner="default_user",
        provider="",
    )
    def query():
        return "SELECT * FROM {{ table }}"

    resource = registrar.state().sorted_list()[0]
    assert resource.definition.query == "SELECT * FROM {{ table." + run + " }}"
    assert resource.variant == variant


def test_register_df_with_variant():
    variant = "test"
    registrar = Registrar()
    run = registrar.get_run()

    @registrar.df_transformation(
        name="test",
        variant=variant,
        owner="default_user",
        provider="",
        inputs=[("name", ""), "name2"],
        tags=[],
        properties={},
    )
    def some_function(x):
        return x

    resource = registrar.state().sorted_list()[0]
    for input in resource.definition.inputs:
        assert input[1] == run
    assert resource.variant == variant


def test_register_trainingset_with_variant():
    variant = "test"
    registrar = Registrar()
    run = registrar.get_run()
    registrar.register_training_set(
        name="test",
        variant=variant,
        owner="default_user",
        features=[("feature", "")],
        label=("label", ""),
        tags=[],
        properties={},
    )
    resource = registrar.state().sorted_list()[0]
    for feature in resource.features:
        assert feature[1] == run
    assert resource.label[1] == run
    assert resource.variant == variant


def test_register_trainingset_string_with_variant():
    variant = "test"
    registrar = Registrar()
    run = registrar.get_run()
    registrar.register_training_set(
        name="test",
        variant=variant,
        owner="default_user",
        features=["feature"],
        label="label",
        tags=[],
        properties={},
    )
    resource = registrar.state().sorted_list()[0]
    for feature in resource.features:
        assert feature[1] == run
    assert resource.label[1] == run
    assert resource.variant == variant



from unittest.mock import Mock

import pytest

from featureform import Client, OnlineProvider, ResourceType, entity
from featureform import (
    ColumnSourceRegistrar,
    FeatureColumnResource,
    LabelColumnResource,
    OfflineProvider,
    SubscriptableTransformation,
    TrainingSetVariant,
)


class TestDelete:
    @pytest.fixture
    def client(self):
        """Fixture to provide a fresh client instance with a mocked stub."""
        client = Client(host="localhost:7878", insecure=True, dry_run=True)
        client._stub = Mock()
        return client

    @pytest.mark.parametrize("method", ["delete", "prune"])
    def test_string_input_without_resource_type(self, client, method):
        """Should raise ValueError when string input has no resource type."""
        with pytest.raises(
            ValueError, match="resource_type must be specified if source is a string"
        ):
            getattr(client, method)("test_source")

    @pytest.mark.parametrize("method", ["delete", "prune"])
    def test_string_input_provider(self, client, method):
        """Should create a valid request for a provider without requiring variant."""
        getattr(client, method)("test_provider", resource_type=ResourceType.PROVIDER)

        stub_method = (
            client._stub.MarkForDeletion
            if method == "delete"
            else client._stub.PruneResource
        )
        stub_method.assert_called_once()

        request = stub_method.call_args[0][0]
        assert request.resource_id.resource.name == "test_provider"
        assert request.resource_id.resource.variant == ""
        assert request.resource_id.resource_type == ResourceType.PROVIDER.to_proto()

    @pytest.mark.parametrize("method", ["delete", "prune"])
    def test_string_input_non_provider_without_variant(self, client, method):
        """Should raise ValueError for non-provider string input without variant."""
        with pytest.raises(
            ValueError, match="variant must be specified for non-provider resources"
        ):
            getattr(client, method)(
                "test_feature", resource_type=ResourceType.FEATURE_VARIANT
            )

    @pytest.mark.parametrize("method", ["delete", "prune"])
    def test_string_input_with_variant(self, client, method):
        """Should create a valid request for string input with a variant."""
        getattr(client, method)(
            "source_variant", variant="v1", resource_type=ResourceType.SOURCE_VARIANT
        )

        stub_method = (
            client._stub.MarkForDeletion
            if method == "delete"
            else client._stub.PruneResource
        )
        stub_method.assert_called_once()

        request = stub_method.call_args[0][0]
        assert request.resource_id.resource.name == "source_variant"
        assert request.resource_id.resource.variant == "v1"
        assert (
            request.resource_id.resource_type == ResourceType.SOURCE_VARIANT.to_proto()
        )

    @pytest.mark.parametrize("method", ["delete", "prune"])
    def test_provider_input(self, client, method):
        """Should create a valid request for a provider object."""
        # Mock provider object
        provider = Mock(spec=OnlineProvider)
        provider.name.return_value = "test_provider"

        getattr(client, method)(provider)

        stub_method = (
            client._stub.MarkForDeletion
            if method == "delete"
            else client._stub.PruneResource
        )
        stub_method.assert_called_once()

        request = stub_method.call_args[0][0]
        assert request.resource_id.resource.name == "test_provider"
        assert request.resource_id.resource.variant == ""
        assert request.resource_id.resource_type == ResourceType.PROVIDER.to_proto()

    @pytest.mark.parametrize("method", ["delete", "prune"])
    def test_non_deletable_resource(self, client, method):
        """Should raise ValueError for a resource that does not implement DeletableResource."""

        # Mock a resource not implementing DeletableResource
        @entity
        class User:
            pass

        with pytest.raises(ValueError, match="resource is not deletable"):
            getattr(client, method)(User)

    @pytest.mark.parametrize(
        "resource_type,variant,name",
        [
            (ResourceType.PROVIDER, None, "test_provider"),
            (ResourceType.SOURCE_VARIANT, "v1", "test_source"),
            (ResourceType.FEATURE_VARIANT, "v1", "test_feature"),
            (ResourceType.LABEL_VARIANT, "v2", "test_label"),
            (ResourceType.TRAININGSET_VARIANT, "v3", "test_trainingset"),
            (ResourceType.TRANSFORMATION, "v4", "test_transformation"),
        ],
    )
    @pytest.mark.parametrize("method", ["delete", "prune"])
    def test_valid_deletable_resources(
        self, client, resource_type, variant, name, method
    ):
        """Should create valid requests for all deletable resource types."""
        if variant:
            getattr(client, method)(name, variant=variant, resource_type=resource_type)
        else:
            getattr(client, method)(name, resource_type=resource_type)

        # Determine the correct stub method
        stub_method = (
            client._stub.MarkForDeletion
            if method == "delete"
            else client._stub.PruneResource
        )
        stub_method.assert_called_once()

        # Validate the request object
        request = stub_method.call_args[0][0]
        assert request.resource_id.resource.name == name
        assert request.resource_id.resource.variant == (variant or "")
        assert request.resource_id.resource_type == resource_type.to_proto()

    @pytest.mark.parametrize(
        "resource_type",
        [
            "USER",
            "ENTITY",
            "ONDEMAND_FEATURE",
            "SCHEDULE",
            "MODEL",
        ],
    )
    @pytest.mark.parametrize("method", ["delete", "prune"])
    def test_invalid_resource_type(self, client, resource_type, method):
        """Should raise ValueError for resource types that are not deletable."""
        with pytest.raises(ValueError, match="resource_type must be deletable"):
            getattr(client, method)(
                "test_source",
                variant="v1",
                resource_type=getattr(ResourceType, resource_type, None),
            )

    @pytest.mark.parametrize(
        "resource_obj,expected_resource_type",
        [
            (Mock(spec=FeatureColumnResource), ResourceType.FEATURE_VARIANT),
            (Mock(spec=SubscriptableTransformation), ResourceType.TRANSFORMATION),
            (Mock(spec=LabelColumnResource), ResourceType.LABEL_VARIANT),
            (Mock(spec=TrainingSetVariant), ResourceType.TRAININGSET_VARIANT),
            (Mock(spec=ColumnSourceRegistrar), ResourceType.SOURCE_VARIANT),
            (Mock(spec=OnlineProvider), ResourceType.PROVIDER),
            (Mock(spec=OfflineProvider), ResourceType.PROVIDER),
        ],
    )
    @pytest.mark.parametrize("method", ["delete", "prune"])
    def test_deletable_resource_objects(
        self, client, resource_obj, expected_resource_type, method
    ):
        """Should create valid requests for all deletable resource objects."""
        # Mock the name_variant and get_resource_type methods for the resource object
        if expected_resource_type == ResourceType.PROVIDER:
            resource_obj.name.return_value = "test_resource"
        else:
            resource_obj.name_variant.return_value = ("test_resource", "v1")
            resource_obj.get_resource_type.return_value = expected_resource_type

        # Call the delete or prune method
        getattr(client, method)(resource_obj)

        # Determine the correct stub method
        stub_method = (
            client._stub.MarkForDeletion
            if method == "delete"
            else client._stub.PruneResource
        )
        stub_method.assert_called_once()

        # Validate the request object
        request = stub_method.call_args[0][0]
        assert request.resource_id.resource.name == "test_resource"
        if expected_resource_type == ResourceType.PROVIDER:
            assert request.resource_id.resource.variant == ""
        else:
            assert request.resource_id.resource.variant == "v1"
        assert request.resource_id.resource_type == expected_resource_type.to_proto()



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

import time
import featureform as ff
from featureform.resources import Model
import os
import pytest


@pytest.fixture(autouse=True)
def before_and_after_each(setup_teardown):
    setup_teardown()
    yield
    setup_teardown()


@pytest.mark.parametrize(
    "provider_source_fxt,is_local,is_insecure",
    [
        pytest.param(
            "hosted_sql_provider_and_source", False, False, marks=pytest.mark.hosted
        ),
        pytest.param(
            "hosted_sql_provider_and_source", False, True, marks=pytest.mark.docker
        ),
    ],
)
def test_simple_search(provider_source_fxt, is_local, is_insecure, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    provider, source, inference_store = request.getfixturevalue(provider_source_fxt)(
        custom_marks
    )

    # Arranges the resources context following the Quickstart pattern
    resource_client = arrange_resources(
        provider, source, inference_store, is_local, is_insecure
    )

    phrase_a_results = resource_client.search("quick", is_local)
    phrase_b_results = resource_client.search("transact", is_local)
    phrase_c_results = resource_client.search("dataset", is_local)
    phrase_d_results = resource_client.search("foo", is_local)

    counts = [
        len(phrase_a_results),
        len(phrase_b_results),
        len(phrase_c_results),
        len(phrase_d_results),
    ]

    assert counts == [4, 3, 1, 0]


@pytest.mark.parametrize(
    "provider_source_fxt,is_local,is_insecure",
    [
        pytest.param(
            "hosted_sql_provider_and_source", False, False, marks=pytest.mark.hosted
        ),
        pytest.param(
            "hosted_sql_provider_and_source", False, True, marks=pytest.mark.docker
        ),
    ],
)
def test_special_character_search(provider_source_fxt, is_local, is_insecure, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    provider, source, inference_store = request.getfixturevalue(provider_source_fxt)(
        custom_marks
    )

    # Arranges the resources context following the Quickstart pattern
    resource_client = arrange_resources(
        provider, source, inference_store, is_local, is_insecure
    )

    phrase_a_results = resource_client.search("qui#ck!", is_local)
    phrase_b_results = resource_client.search("t*ran.saction!", is_local)
    phrase_c_results = resource_client.search("d,ata@se-t", is_local)

    assert (
        len(phrase_a_results) == 4
        and len(phrase_b_results) == 3
        and len(phrase_c_results) == 1
    )


@pytest.mark.parametrize(
    "provider_source_fxt,is_local,is_insecure",
    [
        pytest.param(
            "hosted_sql_provider_and_source", False, False, marks=pytest.mark.hosted
        ),
        pytest.param(
            "hosted_sql_provider_and_source", False, True, marks=pytest.mark.docker
        ),
    ],
)
def test_empty_query_to_search(provider_source_fxt, is_local, is_insecure, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    provider, source, inference_store = request.getfixturevalue(provider_source_fxt)(
        custom_marks
    )

    # Arranges the resources context following the Quickstart pattern
    resource_client = arrange_resources(
        provider, source, inference_store, is_local, is_insecure
    )

    with pytest.raises(Exception, match="query must be string and cannot be empty"):
        phrase = resource_client.search(raw_query="", local=is_local)


@pytest.mark.parametrize(
    "provider_source_fxt,is_local,is_insecure",
    [
        pytest.param(
            "hosted_sql_provider_and_source", False, False, marks=pytest.mark.hosted
        ),
        pytest.param(
            "hosted_sql_provider_and_source", False, True, marks=pytest.mark.docker
        ),
    ],
)
def test_query_type_in_search(provider_source_fxt, is_local, is_insecure, request):
    custom_marks = [
        mark.name for mark in request.node.own_markers if mark.name != "parametrize"
    ]
    provider, source, inference_store = request.getfixturevalue(provider_source_fxt)(
        custom_marks
    )

    # Arranges the resources context following the Quickstart pattern
    resource_client = arrange_resources(
        provider, source, inference_store, is_local, is_insecure
    )

    with pytest.raises(
        TypeError, match="missing 1 required positional argument: 'raw_query'"
    ):
        phrase = resource_client.search(local=is_local)


def arrange_resources(provider, source, online_store, is_local, is_insecure):
    if is_local:

        @provider.df_transformation(
            variant="quickstart", inputs=[("transactions", "quickstart")]
        )
        def average_user_transaction(transactions):
            return transactions.groupby("CustomerID")["TransactionAmount"].mean()

    else:

        @provider.sql_transformation(variant="quickstart")
        def average_user_transaction():
            return "SELECT customerid as user_id, avg(transactionamount) as avg_transaction_amt from {{transactions.quickstart}} GROUP BY user_id"

    user = ff.register_entity("user")
    feature_column = "TransactionAmount" if is_local else "avg_transaction_amt"
    label_column = "IsFraud" if is_local else "isfraud"
    inference_store = provider if is_local else online_store

    average_user_transaction.register_resources(
        entity=user,
        entity_column="CustomerID" if is_local else "user_id",
        inference_store=inference_store,
        features=[
            {
                "name": "avg_transactions",
                "variant": "quickstart",
                "column": feature_column,
                "type": "float32",
            },
        ],
    )

    source.register_resources(
        entity=user,
        entity_column="CustomerID" if is_local else "customerid",
        labels=[
            {
                "name": "fraudulent",
                "variant": "quickstart",
                "column": label_column,
                "type": "bool",
            },
        ],
    )

    resource_client = ff.ResourceClient(local=is_local, insecure=is_insecure)
    resource_client.apply()

    return resource_client



#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
#  Copyright 2024 FeatureForm Inc.
#

import os
import sys
import pytest
import dill
import random
from unittest.mock import MagicMock


sys.path.insert(0, "client/src/")
from featureform.register import (
    ResourceClient,
    DFTransformation,
    SQLTransformation,
    PrimaryData,
    SQLTable,
)
from featureform.resources import (
    FeatureVariant,
    LabelVariant,
    TrainingSetVariant,
    SourceVariant,
    Transformation,
    ResourceColumnMapping,
    ResourceState,
)
from featureform.proto import metadata_pb2 as pb
from featureform.enums import ResourceType, ResourceStatus
from featureform.status_display import (
    DisplayStatus,
    ResourceTableRow,
    StatusDisplayer,
    NUM_DISPLAY_ROWS,
)


def my_func(df):
    return df


pb_no_status = pb.ResourceStatus.Status.NO_STATUS
pb_created = pb.ResourceStatus.Status.CREATED
pb_pending = pb.ResourceStatus.Status.PENDING
pb_ready = pb.ResourceStatus.Status.READY
pb_failed = pb.ResourceStatus.Status.FAILED
sql_query = "SELECT * FROM NONE"
df_query = dill.dumps(my_func)
df_source_text = dill.source.getsource(my_func)
df_name_variants = [("name", "variant")]
primary_table_name = "my_table"

sql_definition_proto = pb.SourceVariant(
    name="",
    transformation=pb.Transformation(
        SQLTransformation=pb.SQLTransformation(query=sql_query)
    ),
)

df_definition_proto = pb.SourceVariant(
    name="",
    transformation=pb.Transformation(
        DFTransformation=pb.DFTransformation(
            query=df_query,
            inputs=[
                pb.NameVariant(name=nv[0], variant=nv[1]) for nv in df_name_variants
            ],
            source_text=df_source_text,
        )
    ),
)

primary_definition_proto = pb.SourceVariant(
    name="",
    primaryData=pb.PrimaryData(table=pb.SQLTable(name=primary_table_name)),
)

sql_definition_obj = SQLTransformation(sql_query)

df_definition_obj = DFTransformation(
    query=df_query, inputs=df_name_variants, source_text=df_source_text
)

primary_definition_obj = PrimaryData(SQLTable(primary_table_name))


# Fetches status from proto for same response that client would give
def get_pb_status(status):
    return pb.ResourceStatus.Status._enum_type.values[status].name


@pytest.mark.parametrize(
    "status,expected,ready",
    [
        (pb_no_status, ResourceStatus.NO_STATUS, False),
        (pb_created, ResourceStatus.CREATED, False),
        (pb_pending, ResourceStatus.PENDING, False),
        (pb_ready, ResourceStatus.READY, True),
        (pb_failed, ResourceStatus.FAILED, False),
    ],
)
def test_feature_status(mocker, status, expected, ready):
    # Environment variable is getting set somewhere when using Makefile
    # Needs further investigation
    os.environ.pop("FEATUREFORM_CERT", None)
    mocker.patch.object(
        ResourceClient,
        "get_feature",
        return_value=FeatureVariant(
            created=None,
            name="name",
            variant="",
            source=("some", "source"),
            value_type="float32",
            entity="entity",
            owner="me",
            provider="provider",
            location=ResourceColumnMapping("", "", ""),
            description="",
            status=get_pb_status(status),
            tags=[],
            properties={},
        ),
    )
    client = ResourceClient("host")
    feature = client.get_feature("", "")
    assert feature.get_status() == expected
    assert feature.status == expected.name
    assert feature.is_ready() == ready


@pytest.mark.parametrize(
    "status,expected,ready",
    [
        (pb_no_status, ResourceStatus.NO_STATUS, False),
        (pb_created, ResourceStatus.CREATED, False),
        (pb_pending, ResourceStatus.PENDING, False),
        (pb_ready, ResourceStatus.READY, True),
        (pb_failed, ResourceStatus.FAILED, False),
    ],
)
def test_label_status(mocker, status, expected, ready):
    mocker.patch.object(
        ResourceClient,
        "get_label",
        return_value=LabelVariant(
            name="name",
            variant="",
            source=("some", "source"),
            value_type="float32",
            entity="entity",
            owner="me",
            provider="provider",
            location=ResourceColumnMapping("", "", ""),
            description="",
            status=get_pb_status(status),
            tags=[],
            properties={},
        ),
    )
    client = ResourceClient("host")
    label = client.get_label("", "")
    assert label.get_status() == expected
    assert label.status == expected.name
    assert label.is_ready() == ready


@pytest.mark.parametrize(
    "status,expected,ready",
    [
        (pb_no_status, ResourceStatus.NO_STATUS, False),
        (pb_created, ResourceStatus.CREATED, False),
        (pb_pending, ResourceStatus.PENDING, False),
        (pb_ready, ResourceStatus.READY, True),
        (pb_failed, ResourceStatus.FAILED, False),
    ],
)
def test_training_set_status(mocker, status, expected, ready):
    mocker.patch.object(
        ResourceClient,
        "get_training_set",
        return_value=TrainingSetVariant(
            created=None,
            name="",
            variant="",
            owner="",
            label=("something", "something"),
            features=[("some", "feature")],
            feature_lags=[],
            description="",
            status=get_pb_status(status),
            tags=[],
            properties={},
        ),
    )
    client = ResourceClient("host")
    ts = client.get_training_set("", "")
    assert ts.get_status() == expected
    assert ts.status == expected.name
    assert ts.is_ready() == ready


@pytest.mark.parametrize(
    "status,expected,ready",
    [
        (pb_no_status, ResourceStatus.NO_STATUS, False),
        (pb_created, ResourceStatus.CREATED, False),
        (pb_pending, ResourceStatus.PENDING, False),
        (pb_ready, ResourceStatus.READY, True),
        (pb_failed, ResourceStatus.FAILED, False),
    ],
)
def test_source_status(mocker, status, expected, ready):
    mocker.patch.object(
        ResourceClient,
        "get_source",
        return_value=SourceVariant(
            created=None,
            name="",
            variant="",
            definition=SQLTransformation(query=""),
            owner="me",
            provider="provider",
            description="",
            status=get_pb_status(status),
            tags=[],
            properties={},
        ),
    )
    client = ResourceClient("host")
    source = client.get_source("", "")
    assert source.get_status() == expected
    assert source.status == expected.name
    assert source.is_ready() == ready


@pytest.mark.parametrize(
    "source,expected",
    [
        (sql_definition_proto, sql_definition_obj),
    ],
)
def test_sql_source_definition_parse(source, expected):
    source_obj = SourceVariant._get_source_definition(source)
    assert source_obj.query == expected.query


@pytest.mark.parametrize(
    "source,expected",
    [
        (df_definition_proto, df_definition_obj),
    ],
)
def test_df_source_definition_parse(source, expected):
    source_obj = SourceVariant._get_source_definition(source)
    assert source_obj.query == expected.query
    assert source_obj.inputs == expected.inputs


@pytest.mark.parametrize(
    "source,expected",
    [
        (primary_definition_proto, primary_definition_obj),
    ],
)
def test_primary_source_definition_parse(source, expected):
    source_obj = SourceVariant._get_source_definition(source)
    assert source_obj.location.name == expected.location.name


@pytest.mark.parametrize(
    "host, resource_type, resource_name, variant, expected_url",
    [
        (
            "localhost:7878",
            ResourceType.FEATURE_VARIANT,
            "featurename",
            "variant",
            "http://localhost/features/featurename?variant=variant",
        ),
        (
            "gcp.featureform.com",
            ResourceType.SOURCE_VARIANT,
            "sourcename",
            "variant",
            "https://gcp.featureform.com/sources/sourcename?variant=variant",
        ),
        (
            "localhost:7878",
            ResourceType.LABEL_VARIANT,
            "labelname",
            "",
            "http://localhost/labels/labelname",
        ),
        (
            "localhost:7878",
            ResourceType.PROVIDER,
            "providername",
            "",
            "http://localhost/providers/providername",
        ),
        (
            "localhost:7878",
            ResourceType.TRANSFORMATION,
            "transformationname",
            "variant",
            "http://localhost/sources/transformationname?variant=variant",
        ),
    ],
)
def test_build_dashboard_url(host, resource_type, resource_name, variant, expected_url):
    url = ResourceState().build_dashboard_url(
        host, resource_type, resource_name, variant
    )
    assert url == expected_url


def test_failed_dashboard_url():
    with pytest.raises(KeyError):
        ResourceState().build_dashboard_url(
            "localhost:7878", "Server", "name", "variant"
        )


displaystatus_feature_pending = DisplayStatus(
    ResourceType.FEATURE_VARIANT, "displaystatus", "feature", "PENDING", ""
)
displaystatus_feature_ready = DisplayStatus(
    ResourceType.FEATURE_VARIANT, "displaystatus", "feature", "READY", ""
)
displaystatus_label_pending = DisplayStatus(
    ResourceType.LABEL_VARIANT, "displaystatus", "label", "PENDING", ""
)
training_set = TrainingSetVariant(
    name="ts2",
    owner="User",
    label=("label", "variant"),
    features=[("feature", "variant")],
    description="description",
    variant="var",
)

feature_variants = []
for i in range(1, 51):
    feature_variant = FeatureVariant(
        created=None,
        name="feature_name",
        variant=f"variant_{i}",
        source=("some", "source"),
        value_type="float32",
        entity="entity",
        owner="me",
        provider="provider",
        location=ResourceColumnMapping("", "", ""),
        description="",
        status="PENDING",
        tags=[],
        properties={},
    )
    feature_variants.append(feature_variant)

label_variants = []
for i in range(50):
    label_variant = LabelVariant(
        name="label_name",
        variant=f"{i}",
        source=("some", "source"),
        value_type="float32",
        entity="entity",
        owner="me",
        provider="provider",
        location=ResourceColumnMapping("", "", ""),
        description="",
        status="PENDING",
        tags=[],
        properties={},
    )
    label_variants.append(label_variant)


@pytest.mark.parametrize(
    "initial_status, resource, new_status, new_error, expected_error",
    [
        (
            displaystatus_feature_pending,
            feature_variants[0],
            ResourceStatus.READY,
            None,
            None,
        ),
        (
            displaystatus_label_pending,
            label_variants[0],
            None,
            "NetworkError",
            None,
        ),
        (
            displaystatus_feature_pending,
            feature_variants[0],
            None,
            None,
            f"No updates provided to status for feature_name",
        ),
        (
            displaystatus_label_pending,
            None,
            None,
            "NetworkError",
            "Resource cannot be empty",
        ),
    ],
)
def test_update_status(initial_status, resource, new_status, new_error, expected_error):
    if not resource:
        with pytest.raises(ValueError, match=expected_error):
            ResourceTableRow(resource, initial_status)
        return
    row = ResourceTableRow(resource, initial_status)
    assert row.get_status() == initial_status
    assert row.name == resource.name
    assert row.get_time_ticks() == 0
    assert row.resource == resource

    if expected_error:
        with pytest.raises(ValueError, match=expected_error):
            row.update_status(new_status=new_status, error=new_error)
    else:
        row.update_status(new_status=new_status, error=new_error)
        updated_status = row.get_status()
        if new_status:
            assert updated_status.status == new_status
        if new_error:
            assert updated_status.error == new_error


def test_update_display_data():
    status_displayer = StatusDisplayer(None, feature_variants)
    expected_success_list = []
    expected_failed_list = []
    status_displayer.setup_display_resources()

    def update_resource_status(resource_table_row):
        random_num = random.randint(0, 2)
        if random_num == 0:
            resource_table_row.update_status(ResourceStatus.READY, None)
            if resource_table_row not in expected_success_list:
                expected_success_list.append(resource_table_row)
        elif random_num == 1:
            resource_table_row.update_status(ResourceStatus.FAILED, "NetworkError")
            if resource_table_row not in expected_failed_list:
                expected_failed_list.append(resource_table_row)
        else:
            resource_table_row.update_status(ResourceStatus.PENDING)

    status_displayer.update_resource_status = MagicMock(
        side_effect=update_resource_status
    )
    for _ in range(10):
        status_displayer.update_display_data()
        assert set(status_displayer.success_list) == set(expected_success_list)
        assert set(status_displayer.failed_list) == set(expected_failed_list)


def test_display_table():
    status_displayer = StatusDisplayer(None, label_variants)
    len_all_resources = len(label_variants)
    expected_display_table = status_displayer.unprocessed_resources[:NUM_DISPLAY_ROWS]

    def mock_update_all_resource_statuses():
        return

    status_displayer.update_all_resource_statuses = MagicMock(
        side_effect=mock_update_all_resource_statuses
    )
    status_displayer.setup_display_resources()
    len_all_resources = len_all_resources - NUM_DISPLAY_ROWS
    assert set(status_displayer.display_table_list) == set(expected_display_table)
    assert len(status_displayer.unprocessed_resources) == len_all_resources

    def mock_update_display_status(resource_table_row: ResourceTableRow):
        pass

    status_displayer.update_resource_status = MagicMock(
        side_effect=mock_update_display_status
    )

    batch = 5
    for index, resource_table_row in enumerate(status_displayer.display_table_list):
        if index >= batch:
            break
        resource_table_row.update_status(ResourceStatus.READY, None)
        len_all_resources = len_all_resources - 1

    status_displayer._process_existing_display_rows()
    assert len(status_displayer.display_table_list) == NUM_DISPLAY_ROWS
    status_displayer._process_existing_display_rows()
    assert len(status_displayer.display_table_list) == NUM_DISPLAY_ROWS - batch
    status_displayer._fill_display_rows_from_unprocessed_list()
    assert len(status_displayer.display_table_list) == NUM_DISPLAY_ROWS
    assert len(status_displayer.unprocessed_resources) == len_all_resources

    batch = 15
    for index, resource_table_row in enumerate(status_displayer.display_table_list):
        if index >= batch:
            break
        resource_table_row.update_status(ResourceStatus.READY, None)

    for index, resource_table_row in enumerate(status_displayer.unprocessed_resources):
        if index >= batch:
            break
        resource_table_row.update_status(ResourceStatus.FAILED, "NetworkError")

    status_displayer._process_existing_display_rows()
    assert len(status_displayer.display_table_list) == NUM_DISPLAY_ROWS
    status_displayer._process_existing_display_rows()
    assert len(status_displayer.display_table_list) == NUM_DISPLAY_ROWS - batch
    status_displayer._fill_display_rows_from_unprocessed_list()
    if (len_all_resources - batch) < batch:
        assert len(
            status_displayer.display_table_list
        ) == NUM_DISPLAY_ROWS + len_all_resources - (2 * batch)
    else:
        assert len(status_displayer.display_table_list) == NUM_DISPLAY_ROWS
    len_all_resources = len_all_resources - (2 * batch)
    assert len(status_displayer.unprocessed_resources) == max(len_all_resources, 0)
